# This file is automatically generated by botocraft.  Do not edit directly.
# mypy: disable-error-code="index, override, assignment, union-attr, misc"
from botocraft.mixins.tags import TagsDictMixin
from .abstract import PrimaryBoto3ModelQuerySet
from collections import OrderedDict
from typing import ClassVar, Type, Optional, Literal, Dict, List, Union, Any, cast
from datetime import datetime
from botocraft.mixins.route53 import HostedZoneModelMixin
from pydantic import Field
from .abstract import (
    Boto3Model,
    ReadonlyBoto3Model,
    PrimaryBoto3Model,
    ReadonlyPrimaryBoto3Model,
    Boto3ModelManager,
    ReadonlyBoto3ModelManager,
)
from functools import cached_property

# ===============
# Managers
# ===============


class HostedZoneManager(Boto3ModelManager):
    service_name: str = "route53"

    def create(
        self,
        model: "HostedZone",
        VPC: Optional["Route53VPC"] = None,
        HostedZoneConfig: Optional["HostedZoneConfig"] = None,
        DelegationSetId: str | None = None,
    ) -> "HostedZone":
        """
        Creates a new public or private hosted zone. You create records in a public
        hosted zone to define how you want to route traffic on the internet for a
        domain, such as example.com, and its subdomains (apex.example.com,
        acme.example.com). You create records in a private hosted zone to define how you
        want to route traffic for a domain and its subdomains within one or more Amazon
        Virtual Private Clouds (Amazon VPCs).

        Args:
            model: The :py:class:`HostedZone` to create.

        Keyword Args:
            VPC: (Private hosted zones only) A complex type that contains information about the Amazon VPC that you're
                associating with this hosted zone.
            HostedZoneConfig: (Optional) A complex type that contains the following optional values:
            DelegationSetId: If you want to associate a reusable delegation set with this hosted zone, the ID that Amazon
                Route 53 assigned to the reusable delegation set when you created it. For more information about reusable delegation
                sets, see `CreateReusableDel egationSet
                <https://docs.aws.amazon.com/Route53/latest/APIReference/API_CreateReusableDelegationSet.html>`_.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            Name=data.get("Name"),
            CallerReference=data.get("CallerReference"),
            VPC=self.serialize(VPC),
            HostedZoneConfig=self.serialize(HostedZoneConfig),
            DelegationSetId=self.serialize(DelegationSetId),
        )
        _response = self.client.create_hosted_zone(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateHostedZoneResponse(**_response)

        self.sessionize(response.HostedZone)
        return cast("HostedZone", response.HostedZone)

    def delete(self, Id: str) -> "HostedZone":
        """
        Deletes a hosted zone.

        Args:
            Id: The ID of the hosted zone you want to delete.
        """
        args: Dict[str, Any] = dict(Id=self.serialize(Id))
        _response = self.client.delete_hosted_zone(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DeleteHostedZoneResponse(**_response)
        return cast("HostedZone", response.ChangeInfo)

    def get(self, Id: str) -> Optional["HostedZone"]:
        """
        Gets information about a specified hosted zone including the four name servers
        assigned to the hosted zone.

        Args:
            Id: The ID of the hosted zone that you want to get information about.
        """
        args: Dict[str, Any] = dict(Id=self.serialize(Id))
        _response = self.client.get_hosted_zone(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = GetHostedZoneResponse(**_response)

        if response and response.HostedZone:
            self.sessionize(response.HostedZone)
            return response.HostedZone
        return None

    def list(
        self,
        *,
        MaxItems: str | None = None,
        DelegationSetId: str | None = None,
        HostedZoneType: Literal["PrivateHostedZone"] | None = None,
    ) -> PrimaryBoto3ModelQuerySet:
        """
        Retrieves a list of the public and private hosted zones that are associated with
        the current Amazon Web Services account. The response includes a ``HostedZones``
        child element for each hosted zone.

        Keyword Args:
            MaxItems: (Optional) The maximum number of hosted zones that you want Amazon Route 53 to return. If you have more
                than ``maxitems`` hosted zones, the value of ``IsTruncated`` in the response is ``true``, and the value of
                ``NextMarker`` is the hosted zone ID of the first hosted zone that Route 53 will return if you submit another
                request.
            DelegationSetId: If you're using reusable delegation sets and you want to list all of the hosted zones that are
                associated with a reusable delegation set, specify the ID of that reusable delegation set.
            HostedZoneType: (Optional) Specifies if the hosted zone is private.
        """
        paginator = self.client.get_paginator("list_hosted_zones")
        args: Dict[str, Any] = dict(
            MaxItems=self.serialize(MaxItems),
            DelegationSetId=self.serialize(DelegationSetId),
            HostedZoneType=self.serialize(HostedZoneType),
        )
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results = []
        for _response in response_iterator:
            if list(_response.keys()) == ["ResponseMetadata"]:
                break
            if "ResponseMetadata" in _response:
                del _response["ResponseMetadata"]
            response = ListHostedZonesResponse(**_response)
            if response.HostedZones:
                results.extend(response.HostedZones)
            else:
                if getattr(response, "NextToken", None):
                    continue
                break
        self.sessionize(results)
        if results and isinstance(results[0], Boto3Model):
            return PrimaryBoto3ModelQuerySet(results)
        return results

    def list_records(
        self,
        HostedZoneId: str,
        *,
        StartRecordName: str | None = None,
        StartRecordType: Literal["SOA", "A", "TXT", "NS", "CNAME", "MX", "NAPTR", "PTR", "SRV", "SPF", "AAAA", "CAA", "DS", "TLSA", "SSHFP", "SVCB", "HTTPS"] | None = None,
        StartRecordIdentifier: str | None = None,
        MaxItems: str | None = None,
    ) -> List["Route53ResourceRecordSet"]:
        """
        Lists the resource record sets in a specified hosted zone.

        Args:
            HostedZoneId: The ID of the hosted zone that contains the resource record sets that you want to list.

        Keyword Args:
            StartRecordName: The first name in the lexicographic ordering of resource record sets that you want to list. If the
                specified record name doesn't exist, the results begin with the first resource record set that has a name greater
                than the value of ``name``.
            StartRecordType: The type of resource record set to begin the record listing from.
            StartRecordIdentifier: *Resource record sets that have a routing policy other than simple:* If results were
                truncated for a given DNS name and   type, specify the value of ``NextRecordIdentifier`` from the previous response
                to get the next resource record set that   has the current DNS name and type.
            MaxItems: (Optional) The maximum number of resource records sets to include in the response body for this request.
                If the response includes more than ``maxitems`` resource record sets, the value of the ``IsTruncated`` element in
                the response is ``true``, and the values of the ``NextRecordName`` and ``NextRecordType`` elements in the response
                identify the first resource record set in the next group of ``maxitems`` resource record sets.
        """
        paginator = self.client.get_paginator("list_resource_record_sets")
        args: Dict[str, Any] = dict(
            HostedZoneId=self.serialize(HostedZoneId),
            StartRecordName=self.serialize(StartRecordName),
            StartRecordType=self.serialize(StartRecordType),
            StartRecordIdentifier=self.serialize(StartRecordIdentifier),
            MaxItems=self.serialize(MaxItems),
        )
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )

        results: List["Route53ResourceRecordSet"] = []

        for _response in response_iterator:
            response = ListResourceRecordSetsResponse(**_response)
            if response.ResourceRecordSets is not None:
                results.extend(response.ResourceRecordSets)
            else:
                break

        self.sessionize(results)
        return cast("List[Route53ResourceRecordSet]", results)

    def list_by_name(
        self,
        *,
        DNSName: str | None = None,
        HostedZoneId: str | None = None,
        MaxItems: str | None = None,
    ) -> List["HostedZone"]:
        """
        Retrieves a list of your hosted zones in lexicographic order. The response
        includes a ``HostedZones`` child element for each hosted zone created by the
        current Amazon Web Services account.

        Keyword Args:
            DNSName: (Optional) For your first request to ``ListHostedZonesByName``, include the ``dnsname`` parameter only if
                you want to specify the name of the first hosted zone in the response. If you don't include the ``dnsname``
                parameter, Amazon Route 53 returns all of the hosted zones that were created by the current Amazon Web Services
                account, in ASCII order. For subsequent requests, include both ``dnsname`` and ``hostedzoneid`` parameters. For
                ``dnsname``, specify the value of ``NextDNSName`` from the previous response.
            HostedZoneId: (Optional) For your first request to ``ListHostedZonesByName``, do not include the ``hostedzoneid``
                parameter.
            MaxItems: The maximum number of hosted zones to be included in the response body for this request. If you have more
                than ``maxitems`` hosted zones, then the value of the ``IsTruncated`` element in the response is true, and the
                values of ``NextDNSName`` and ``NextHostedZoneId`` specify the first hosted zone in the next group of ``maxitems``
                hosted zones.
        """
        args: Dict[str, Any] = dict(
            DNSName=self.serialize(DNSName),
            HostedZoneId=self.serialize(HostedZoneId),
            MaxItems=self.serialize(MaxItems),
        )
        _response = self.client.list_hosted_zones_by_name(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ListHostedZonesByNameResponse(**_response)

        results: List["HostedZone"] = None
        if response is not None:
            results = response.HostedZones

        self.sessionize(results)
        return cast("List[HostedZone]", results)

    def list_by_vpc(
        self,
        *,
        DNSName: str | None = None,
        HostedZoneId: str | None = None,
        MaxItems: str | None = None,
    ) -> List["HostedZone"]:
        """
        Retrieves a list of your hosted zones in lexicographic order. The response
        includes a ``HostedZones`` child element for each hosted zone created by the
        current Amazon Web Services account.

        Keyword Args:
            DNSName: (Optional) For your first request to ``ListHostedZonesByName``, include the ``dnsname`` parameter only if
                you want to specify the name of the first hosted zone in the response. If you don't include the ``dnsname``
                parameter, Amazon Route 53 returns all of the hosted zones that were created by the current Amazon Web Services
                account, in ASCII order. For subsequent requests, include both ``dnsname`` and ``hostedzoneid`` parameters. For
                ``dnsname``, specify the value of ``NextDNSName`` from the previous response.
            HostedZoneId: (Optional) For your first request to ``ListHostedZonesByName``, do not include the ``hostedzoneid``
                parameter.
            MaxItems: The maximum number of hosted zones to be included in the response body for this request. If you have more
                than ``maxitems`` hosted zones, then the value of the ``IsTruncated`` element in the response is true, and the
                values of ``NextDNSName`` and ``NextHostedZoneId`` specify the first hosted zone in the next group of ``maxitems``
                hosted zones.
        """
        args: Dict[str, Any] = dict(
            DNSName=self.serialize(DNSName),
            HostedZoneId=self.serialize(HostedZoneId),
            MaxItems=self.serialize(MaxItems),
        )
        _response = self.client.list_hosted_zones_by_name(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ListHostedZonesByNameResponse(**_response)

        results: List["HostedZone"] = None
        if response is not None:
            results = response.HostedZones

        self.sessionize(results)
        return cast("List[HostedZone]", results)

    def update_comment(self, Id: str, Comment: str) -> "HostedZone":
        """
        Updates the comment for a specified hosted zone.

        Args:
            Id: The ID for the hosted zone that you want to update the comment for.
            Comment: The new comment for the hosted zone. If you don't specify a value for ``Comment``, Amazon Route 53 deletes
                the existing value of the ``Comment`` element, if any.
        """
        args: Dict[str, Any] = dict(
            Id=self.serialize(Id), Comment=self.serialize(Comment)
        )
        _response = self.client.update_hosted_zone_comment(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = UpdateHostedZoneCommentResponse(**_response)

        results: "HostedZone" = None
        if response is not None:
            results = response.HostedZone

        self.sessionize(results)
        return cast("HostedZone", results)

    def count(
        self,
    ) -> int:
        """
        Retrieves the number of hosted zones that are associated with the current Amazon
        Web Services account.
        """
        args: Dict[str, Any] = dict()
        _response = self.client.get_hosted_zone_count(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = GetHostedZoneCountResponse(**_response)

        results: int = None
        if response is not None:
            results = response.HostedZoneCount

        self.sessionize(results)
        return cast("int", results)

    def limits(
        self,
        Type: Literal["MAX_RRSETS_BY_ZONE", "MAX_VPCS_ASSOCIATED_BY_ZONE"],
        HostedZoneId: str,
    ) -> "GetHostedZoneLimitResponse":
        """
        Gets the specified limit for a specified hosted zone, for example, the maximum
        number of records that you can create in the hosted zone.

        Args:
            Type: The limit that you want to get. Valid values include the following:
            HostedZoneId: The ID of the hosted zone that you want to get a limit for.
        """
        args: Dict[str, Any] = dict(
            Type=self.serialize(Type), HostedZoneId=self.serialize(HostedZoneId)
        )
        _response = self.client.get_hosted_zone_limit(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = GetHostedZoneLimitResponse(**_response)

        results: "GetHostedZoneLimitResponse" = None
        if response is not None:
            results = response

        self.sessionize(results)
        return cast("GetHostedZoneLimitResponse", results)

    def associate_vpc(
        self, HostedZoneId: str, VPC: "Route53VPC", *, Comment: str | None = None
    ) -> "ChangeInfo":
        """
        Associates an Amazon VPC with a private hosted zone.

        Args:
            HostedZoneId: The ID of the private hosted zone that you want to associate an Amazon VPC with.
            VPC: A complex type that contains information about the VPC that you want to associate with a private hosted zone.

        Keyword Args:
            Comment: *Optional:* A comment about the association request.
        """
        args: Dict[str, Any] = dict(
            HostedZoneId=self.serialize(HostedZoneId),
            VPC=self.serialize(VPC),
            Comment=self.serialize(Comment),
        )
        _response = self.client.associate_vpc_with_hosted_zone(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = AssociateVPCWithHostedZoneResponse(**_response)

        results: "ChangeInfo" = None
        if response is not None:
            results = response.ChangeInfo

        self.sessionize(results)
        return cast("ChangeInfo", results)

    def disassociate_vpc(
        self, HostedZoneId: str, VPC: "Route53VPC", *, Comment: str | None = None
    ) -> "ChangeInfo":
        """
        Disassociates an Amazon Virtual Private Cloud (Amazon VPC) from an Amazon Route 53 private hosted zone. Note the
        following:

        Args:
            HostedZoneId: The ID of the private hosted zone that you want to disassociate a VPC from.
            VPC: A complex type that contains information about the VPC that you're disassociating from the specified hosted
                zone.

        Keyword Args:
            Comment: *Optional:* A comment about the disassociation request.

        """
        args: Dict[str, Any] = dict(
            HostedZoneId=self.serialize(HostedZoneId),
            VPC=self.serialize(VPC),
            Comment=self.serialize(Comment),
        )
        _response = self.client.disassociate_vpc_from_hosted_zone(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DisassociateVPCFromHostedZoneResponse(**_response)

        results: "ChangeInfo" = None
        if response is not None:
            results = response.ChangeInfo

        self.sessionize(results)
        return cast("ChangeInfo", results)


class Route53VPCManager(Boto3ModelManager):
    service_name: str = "route53"

    def associate(
        self, HostedZoneId: str, VPC: "Route53VPC", *, Comment: str | None = None
    ) -> "ChangeInfo":
        """
        Associates an Amazon VPC with a private hosted zone.

        Args:
            HostedZoneId: The ID of the private hosted zone that you want to associate an Amazon VPC with.
            VPC: A complex type that contains information about the VPC that you want to associate with a private hosted zone.

        Keyword Args:
            Comment: *Optional:* A comment about the association request.
        """
        args: Dict[str, Any] = dict(
            HostedZoneId=self.serialize(HostedZoneId),
            VPC=self.serialize(VPC),
            Comment=self.serialize(Comment),
        )
        _response = self.client.associate_vpc_with_hosted_zone(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = AssociateVPCWithHostedZoneResponse(**_response)

        results: "ChangeInfo" = None
        if response is not None:
            results = response.ChangeInfo

        self.sessionize(results)
        return cast("ChangeInfo", results)

    def disassociate(
        self, HostedZoneId: str, VPC: "Route53VPC", *, Comment: str | None = None
    ) -> "ChangeInfo":
        """
        Disassociates an Amazon Virtual Private Cloud (Amazon VPC) from an Amazon Route 53 private hosted zone. Note the
        following:

        Args:
            HostedZoneId: The ID of the private hosted zone that you want to disassociate a VPC from.
            VPC: A complex type that contains information about the VPC that you're disassociating from the specified hosted
                zone.

        Keyword Args:
            Comment: *Optional:* A comment about the disassociation request.

        """
        args: Dict[str, Any] = dict(
            HostedZoneId=self.serialize(HostedZoneId),
            VPC=self.serialize(VPC),
            Comment=self.serialize(Comment),
        )
        _response = self.client.disassociate_vpc_from_hosted_zone(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DisassociateVPCFromHostedZoneResponse(**_response)

        results: "ChangeInfo" = None
        if response is not None:
            results = response.ChangeInfo

        self.sessionize(results)
        return cast("ChangeInfo", results)

    def list_hosted_zones(
        self,
        VPCId: str,
        VPCRegion: Literal[
            "us-east-1",
            "us-east-2",
            "us-west-1",
            "us-west-2",
            "eu-west-1",
            "eu-west-2",
            "eu-west-3",
            "eu-central-1",
            "eu-central-2",
            "ap-east-1",
            "me-south-1",
            "us-gov-west-1",
            "us-gov-east-1",
            "us-iso-east-1",
            "us-iso-west-1",
            "us-isob-east-1",
            "me-central-1",
            "ap-southeast-1",
            "ap-southeast-2",
            "ap-southeast-3",
            "ap-south-1",
            "ap-south-2",
            "ap-northeast-1",
            "ap-northeast-2",
            "ap-northeast-3",
            "eu-north-1",
            "sa-east-1",
            "ca-central-1",
            "cn-north-1",
            "cn-northwest-1",
            "af-south-1",
            "eu-south-1",
            "eu-south-2",
            "ap-southeast-4",
            "il-central-1",
            "ca-west-1",
            "ap-southeast-5",
            "mx-central-1",
            "us-isof-south-1",
            "us-isof-east-1",
            "ap-southeast-7",
            "ap-east-2",
            "eu-isoe-west-1",
            "ap-southeast-6",
        ],
        *,
        MaxItems: str | None = None,
        NextToken: str | None = None,
    ) -> List["HostedZoneSummary"]:
        """
        Lists all the private hosted zones that a specified VPC is associated with, regardless of which Amazon Web Services
        account or Amazon Web Services service owns the hosted zones. The ``HostedZoneOwner`` structure in the response contains
        one of the following values:

        Args:
            VPCId: The ID of the Amazon VPC that you want to list hosted zones for.
            VPCRegion: For the Amazon VPC that you specified for ``VPCId``, the Amazon Web Services Region that you created the
                VPC in.

        Keyword Args:
            MaxItems: (Optional) The maximum number of hosted zones that you want Amazon Route 53 to return. If the specified
                VPC is associated with more than ``MaxItems`` hosted zones, the response includes a ``NextToken`` element.
                ``NextToken`` contains an encrypted token that identifies the first hosted zone that Route 53 will return if you
                submit another request.
            NextToken: If the previous response included a ``NextToken`` element, the specified VPC is associated with more
                hosted zones. To get more hosted zones, submit another ``ListHostedZonesByVPC`` request.

        """
        args: Dict[str, Any] = dict(
            VPCId=self.serialize(VPCId),
            VPCRegion=self.serialize(VPCRegion),
            MaxItems=self.serialize(MaxItems),
            NextToken=self.serialize(NextToken),
        )
        _response = self.client.list_hosted_zones_by_vpc(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ListHostedZonesByVPCResponse(**_response)

        results: List["HostedZoneSummary"] = None
        if response is not None:
            results = response.HostedZoneSummaries

        self.sessionize(results)
        return cast("List[HostedZoneSummary]", results)

    def list_authorizations(self, HostedZoneId: str) -> List["Route53VPC"]:
        """
        Gets a list of the VPCs that were created by other accounts and that can be
        associated with a specified hosted zone because you've submitted one or more
        ``CreateVPCAssociationAuthorization`` requests.

        Args:
            HostedZoneId: The ID of the hosted zone for which you want a list of VPCs that can be associated with the hosted
                zone.
        """
        paginator = self.client.get_paginator("list_vpc_association_authorizations")
        args: Dict[str, Any] = dict(HostedZoneId=self.serialize(HostedZoneId))
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )

        results: List["Route53VPC"] = []

        for _response in response_iterator:
            response = ListVPCAssociationAuthorizationsResponse(**_response)
            if response.VPCs is not None:
                results.extend(response.VPCs)
            else:
                break

        self.sessionize(results)
        return cast("List[Route53VPC]", results)

    def create_authorization(
        self, HostedZoneId: str, VPC: "Route53VPC"
    ) -> "Route53VPC":
        """
        Authorizes the Amazon Web Services account that created a specified VPC to
        submit an ``AssociateVPCWithHostedZone`` request to associate the VPC with a
        specified hosted zone that was created by a different account. To submit a
        ``CreateVPCAssociationAuthorization`` request, you must use the account that
        created the hosted zone. After you authorize the association, use the account
        that created the VPC to submit an ``AssociateVPCWithHostedZone`` request.

        Args:
            HostedZoneId: The ID of the private hosted zone that you want to authorize associating a VPC with.
            VPC: A complex type that contains the VPC ID and region for the VPC that you want to authorize associating with your
                hosted zone.
        """
        args: Dict[str, Any] = dict(
            HostedZoneId=self.serialize(HostedZoneId), VPC=self.serialize(VPC)
        )
        _response = self.client.create_vpc_association_authorization(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateVPCAssociationAuthorizationResponse(**_response)

        results: "Route53VPC" = None
        if response is not None:
            results = response.VPC

        self.sessionize(results)
        return cast("Route53VPC", results)

    def delete_authorization(
        self, HostedZoneId: str, VPC: "Route53VPC"
    ) -> "DeleteVPCAssociationAuthorizationResponse":
        """
        Removes authorization to submit an ``AssociateVPCWithHostedZone`` request to
        associate a specified VPC with a hosted zone that was created by a different
        account. You must use the account that created the hosted zone to submit a
        ``DeleteVPCAssociationAuthorization`` request.

        Args:
            HostedZoneId: When removing authorization to associate a VPC that was created by one Amazon Web Services account
                with a hosted zone that was created with a different Amazon Web Services account, the ID of the hosted zone.
            VPC: When removing authorization to associate a VPC that was created by one Amazon Web Services account with a
                hosted zone that was created with a different Amazon Web Services account, a complex type that includes the ID and
                region of the VPC.
        """
        args: Dict[str, Any] = dict(
            HostedZoneId=self.serialize(HostedZoneId), VPC=self.serialize(VPC)
        )
        _response = self.client.delete_vpc_association_authorization(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DeleteVPCAssociationAuthorizationResponse(**_response)

        results: "DeleteVPCAssociationAuthorizationResponse" = None
        if response is not None:
            results = response

        self.sessionize(results)
        return cast("DeleteVPCAssociationAuthorizationResponse", results)


class Route53CidrCollectionManager(Boto3ModelManager):
    service_name: str = "route53"

    def create(
        self, model: "Route53CidrCollection", CallerReference: str
    ) -> "CreateCidrCollectionResponse":
        """
        Creates a CIDR collection in the current Amazon Web Services account.

        Args:
            model: The :py:class:`CidrCollection` to create.
            CallerReference: A client-specific token that allows requests to be securely retried so that the intended outcome
                will only occur once, retries receive a similar response, and there are no additional edge cases to handle.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            Name=data.get("Name"), CallerReference=self.serialize(CallerReference)
        )
        _response = self.client.create_cidr_collection(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateCidrCollectionResponse(**_response)

        self.sessionize(response)
        return cast("CreateCidrCollectionResponse", response)

    def delete(self, Id: str) -> None:
        """
        Deletes a CIDR collection in the current Amazon Web Services account. The
        collection must be empty before it can be deleted.

        Args:
            Id: The UUID of the collection to delete.
        """
        args: Dict[str, Any] = dict(Id=self.serialize(Id))
        self.client.delete_cidr_collection(
            **{k: v for k, v in args.items() if v is not None}
        )

    def list(
        self,
    ) -> PrimaryBoto3ModelQuerySet:
        """
        Returns a paginated list of CIDR collections in the Amazon Web Services account
        (metadata only).
        """
        paginator = self.client.get_paginator("list_cidr_collections")
        args: Dict[str, Any] = dict()
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results = []
        for _response in response_iterator:
            if list(_response.keys()) == ["ResponseMetadata"]:
                break
            if "ResponseMetadata" in _response:
                del _response["ResponseMetadata"]
            response = ListCidrCollectionsResponse(**_response)
            if response.CidrCollections:
                results.extend(response.CidrCollections)
            else:
                if getattr(response, "NextToken", None):
                    continue
                break
        self.sessionize(results)
        if results and isinstance(results[0], Boto3Model):
            return PrimaryBoto3ModelQuerySet(results)
        return results

    def update(
        self, model: "Route53CidrCollection", CollectionVersion: int | None = None
    ) -> str:
        """
        Creates, changes, or deletes CIDR blocks within a collection. Contains
        authoritative IP information mapping blocks to one or multiple locations.

        Args:
            model: The :py:class:`CidrCollection` to update.

        Keyword Args:
            CollectionVersion: A sequential counter that Amazon Route 53 sets to 1 when you create a collection and increments
                it by 1 each time you update the collection.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            Id=data.get("Id"),
            Changes=data.get("Changes"),
            CollectionVersion=self.serialize(CollectionVersion),
        )
        _response = self.client.change_cidr_collection(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ChangeCidrCollectionResponse(**_response)

        self.sessionize(response.Id)
        return cast("str", response.Id)


class CidrBlockSummaryManager(Boto3ModelManager):
    service_name: str = "route53"

    def list(
        self, CollectionId: str, *, LocationName: str | None = None
    ) -> PrimaryBoto3ModelQuerySet:
        """
        Returns a paginated list of location objects and their CIDR blocks.

        Args:
            CollectionId: The UUID of the CIDR collection.

        Keyword Args:
            LocationName: The name of the CIDR collection location.
        """
        paginator = self.client.get_paginator("list_cidr_blocks")
        args: Dict[str, Any] = dict(
            CollectionId=self.serialize(CollectionId),
            LocationName=self.serialize(LocationName),
        )
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results = []
        for _response in response_iterator:
            if list(_response.keys()) == ["ResponseMetadata"]:
                break
            if "ResponseMetadata" in _response:
                del _response["ResponseMetadata"]
            response = ListCidrBlocksResponse(**_response)
            if response.CidrBlocks:
                results.extend(response.CidrBlocks)
            else:
                if getattr(response, "NextToken", None):
                    continue
                break
        self.sessionize(results)
        if results and isinstance(results[0], Boto3Model):
            return PrimaryBoto3ModelQuerySet(results)
        return results


class Route53QueryLoggingConfigManager(Boto3ModelManager):
    service_name: str = "route53"

    def create(self, model: "Route53QueryLoggingConfig") -> "Route53QueryLoggingConfig":
        """
        Creates a configuration for DNS query logging. After you create a query logging
        configuration, Amazon Route 53 begins to publish log data to an Amazon
        CloudWatch Logs log group.

        Args:
            model: The :py:class:`QueryLoggingConfig` to create.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            HostedZoneId=data.get("HostedZoneId"),
            CloudWatchLogsLogGroupArn=data.get("CloudWatchLogsLogGroupArn"),
        )
        _response = self.client.create_query_logging_config(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateQueryLoggingConfigResponse(**_response)

        self.sessionize(response.QueryLoggingConfig)
        return cast("Route53QueryLoggingConfig", response.QueryLoggingConfig)

    def delete(self, Id: str) -> None:
        """
        Deletes a configuration for DNS query logging. If you delete a configuration,
        Amazon Route 53 stops sending query logs to CloudWatch Logs. Route 53 doesn't
        delete any logs that are already in CloudWatch Logs.

        Args:
            Id: The ID of the configuration that you want to delete.
        """
        args: Dict[str, Any] = dict(Id=self.serialize(Id))
        self.client.delete_query_logging_config(
            **{k: v for k, v in args.items() if v is not None}
        )

    def get(self, Id: str) -> Optional["Route53QueryLoggingConfig"]:
        """
        Gets information about a specified configuration for DNS query logging.

        Args:
            Id: The ID of the configuration for DNS query logging that you want to get information about.
        """
        args: Dict[str, Any] = dict(Id=self.serialize(Id))
        _response = self.client.get_query_logging_config(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = GetQueryLoggingConfigResponse(**_response)

        if response and response.QueryLoggingConfig:
            self.sessionize(response.QueryLoggingConfig)
            return response.QueryLoggingConfig
        return None

    def list(self, *, HostedZoneId: str | None = None) -> PrimaryBoto3ModelQuerySet:
        """
        Lists the configurations for DNS query logging that are associated with the
        current Amazon Web Services account or the configuration that is associated with
        a specified hosted zone.

        Keyword Args:
            HostedZoneId: (Optional) If you want to list the query logging configuration that is associated with a hosted zone,
                specify the ID in ``HostedZoneId``.
        """
        paginator = self.client.get_paginator("list_query_logging_configs")
        args: Dict[str, Any] = dict(HostedZoneId=self.serialize(HostedZoneId))
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results = []
        for _response in response_iterator:
            if list(_response.keys()) == ["ResponseMetadata"]:
                break
            if "ResponseMetadata" in _response:
                del _response["ResponseMetadata"]
            response = ListQueryLoggingConfigsResponse(**_response)
            if response.QueryLoggingConfigs:
                results.extend(response.QueryLoggingConfigs)
            else:
                if getattr(response, "NextToken", None):
                    continue
                break
        self.sessionize(results)
        if results and isinstance(results[0], Boto3Model):
            return PrimaryBoto3ModelQuerySet(results)
        return results


class Route53ResourceRecordSetManager(Boto3ModelManager):
    service_name: str = "route53"

    def change(self, HostedZoneId: str, ChangeBatch: "ChangeBatch") -> "ChangeInfo":
        """
        Creates, changes, or deletes a resource record set, which contains authoritative
        DNS information for a specified domain name or subdomain name. For example, you
        can use ``ChangeResourceRecordSets`` to create a resource record set that routes
        traffic for test.example.com to a web server that has an IP address of
        192.0.2.44.

        Args:
            HostedZoneId: The ID of the hosted zone that contains the resource record sets that you want to change.
            ChangeBatch: A complex type that contains an optional comment and the ``Changes`` element.
        """
        args: Dict[str, Any] = dict(
            HostedZoneId=self.serialize(HostedZoneId),
            ChangeBatch=self.serialize(ChangeBatch),
        )
        _response = self.client.change_resource_record_sets(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ChangeResourceRecordSetsResponse(**_response)

        results: "ChangeInfo" = None
        if response is not None:
            results = response.ChangeInfo

        self.sessionize(results)
        return cast("ChangeInfo", results)

    def list(
        self,
        HostedZoneId: str,
        *,
        StartRecordName: str | None = None,
        StartRecordType: Literal["SOA", "A", "TXT", "NS", "CNAME", "MX", "NAPTR", "PTR", "SRV", "SPF", "AAAA", "CAA", "DS", "TLSA", "SSHFP", "SVCB", "HTTPS"] | None = None,
        StartRecordIdentifier: str | None = None,
        MaxItems: str | None = None,
    ) -> PrimaryBoto3ModelQuerySet:
        """
        Lists the resource record sets in a specified hosted zone.

        Args:
            HostedZoneId: The ID of the hosted zone that contains the resource record sets that you want to list.

        Keyword Args:
            StartRecordName: The first name in the lexicographic ordering of resource record sets that you want to list. If the
                specified record name doesn't exist, the results begin with the first resource record set that has a name greater
                than the value of ``name``.
            StartRecordType: The type of resource record set to begin the record listing from.
            StartRecordIdentifier: *Resource record sets that have a routing policy other than simple:* If results were
                truncated for a given DNS name and   type, specify the value of ``NextRecordIdentifier`` from the previous response
                to get the next resource record set that   has the current DNS name and type.
            MaxItems: (Optional) The maximum number of resource records sets to include in the response body for this request.
                If the response includes more than ``maxitems`` resource record sets, the value of the ``IsTruncated`` element in
                the response is ``true``, and the values of the ``NextRecordName`` and ``NextRecordType`` elements in the response
                identify the first resource record set in the next group of ``maxitems`` resource record sets.
        """
        paginator = self.client.get_paginator("list_resource_record_sets")
        args: Dict[str, Any] = dict(
            HostedZoneId=self.serialize(HostedZoneId),
            StartRecordName=self.serialize(StartRecordName),
            StartRecordType=self.serialize(StartRecordType),
            StartRecordIdentifier=self.serialize(StartRecordIdentifier),
            MaxItems=self.serialize(MaxItems),
        )
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results = []
        for _response in response_iterator:
            if list(_response.keys()) == ["ResponseMetadata"]:
                break
            if "ResponseMetadata" in _response:
                del _response["ResponseMetadata"]
            response = ListResourceRecordSetsResponse(**_response)
            if response.ResourceRecordSets:
                results.extend(response.ResourceRecordSets)
            else:
                if getattr(response, "NextToken", None):
                    continue
                break
        self.sessionize(results)
        if results and isinstance(results[0], Boto3Model):
            return PrimaryBoto3ModelQuerySet(results)
        return results


# ==============
# Service Models
# ==============


class HostedZoneConfig(Boto3Model):
    """
    A complex type that contains an optional comment about your hosted zone.

    If you don't want to specify a comment, omit
    both the ``HostedZoneConfig`` and ``Comment`` elements.
    """

    Comment: str | None = None
    """
    Any comments that you want to include about the hosted zone.
    """
    PrivateZone: bool | None = None
    """
    A value that indicates whether this is a private hosted zone.
    """


class Route53LinkedService(Boto3Model):
    """
    If a health check or hosted zone was created by another service, ``LinkedService``
    is a complex type that describes the service that created the resource.

    When a resource is created by another service, you can't edit or delete it using
    Amazon Route 53.
    """

    ServicePrincipal: str | None = None
    """
    If the health check or hosted zone was created by another service, the service that
    created the resource.

    When a resource is created by another service, you can't edit or delete it using
    Amazon Route 53.
    """
    Description: str | None = None
    """
    If the health check or hosted zone was created by another service, an optional
    description that can be provided by the other service.

    When a resource is created by another service, you can't edit or delete it using
    Amazon Route 53.
    """


class HostedZone(HostedZoneModelMixin, PrimaryBoto3Model):
    """
    A complex type that contains general information about the hosted zone.
    """

    manager_class: ClassVar[Type[Boto3ModelManager]] = HostedZoneManager

    Id: str = Field(frozen=True)
    """
    The ID that Amazon Route 53 assigned to the hosted zone when you created it.
    """
    Name: str
    """
    The name of the domain.

    For public hosted zones, this is the name that you have registered with your DNS
    registrar.
    """
    CallerReference: str
    """
    The value that you specified for ``CallerReference`` when you created the hosted
    zone.
    """
    Config: HostedZoneConfig = Field(default=None, frozen=True)
    """
    A complex type that includes the ``Comment`` and ``PrivateZone`` elements.

    If you omitted the ``HostedZoneConfig`` and
    ``Comment`` elements from the request, the ``Config`` and ``Comment`` elements don't appear in the response.
    """
    ResourceRecordSetCount: int = Field(default=None, frozen=True)
    """
    The number of resource record sets in the hosted zone.
    """
    LinkedService: Route53LinkedService = Field(default=None, frozen=True)
    """
    If the hosted zone was created by another service, the service that created the
    hosted zone.

    When a hosted zone is created by another service, you can't edit or delete it using
    Route 53.
    """

    @property
    def pk(self) -> str | None:
        """
        Return the primary key of the model.   This is the value of the :py:attr:`Id`
        attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.Id

    @property
    def name(self) -> str | None:
        """
        Return the name of the model.   This is the value of the :py:attr:`Name`
        attribute.

        Returns:
            The name of the model instance.
        """
        return self.Name

    def __hash__(self) -> int:
        """
        Return the hash of the model.

        This is the value of the
        :py:attr:`Id` attribute.
        """
        return hash(self.Id)

    @cached_property
    def records(self) -> List["Route53ResourceRecordSet"] | None:
        """
        Return the records in the hosted zone.

        .. note::

            The output of this property is cached on the model instance, so
            calling this multiple times will not result in multiple calls to the
            AWS API.   If you need a fresh copy of the data, you can re-get the
            model instance from the manager.
        """

        try:
            pk = OrderedDict(
                {
                    "HostedZoneId": self.Id,
                }
            )
        except AttributeError:
            return []
        return Route53ResourceRecordSet.objects.using(self.session).list(**pk)  # type: ignore[arg-type]

    def update_comment(self, Comment: str) -> "HostedZone":
        """
        Update the comment for the hosted zone.

        Args:
            Comment: The new comment for the hosted zone. If you don't specify a value for ``Comment``, Amazon Route 53 deletes
                the existing value of the ``Comment`` element, if any.
        """

        return (
            cast("HostedZoneManager", self.objects)  # type: ignore[attr-defined]
            .using(self.session)
            .update_comment(
                cast("str", self.Id),
                Comment,
            )
        )


class Route53VPC(PrimaryBoto3Model):
    """
    (Private hosted zones only) A complex type that contains information about an Amazon
    VPC.

    If you associate a private hosted zone with an Amazon VPC when you make a
    `CreateHostedZone <https://docs.aws.amazon.com/Route53/latest/APIReference/API_CreateHostedZone.html>`_
    request, the
    following parameters are also required.
    """

    manager_class: ClassVar[Type[Boto3ModelManager]] = Route53VPCManager

    VPCRegion: Literal["us-east-1", "us-east-2", "us-west-1", "us-west-2", "eu-west-1", "eu-west-2", "eu-west-3", "eu-central-1", "eu-central-2", "ap-east-1", "me-south-1", "us-gov-west-1", "us-gov-east-1", "us-iso-east-1", "us-iso-west-1", "us-isob-east-1", "me-central-1", "ap-southeast-1", "ap-southeast-2", "ap-southeast-3", "ap-south-1", "ap-south-2", "ap-northeast-1", "ap-northeast-2", "ap-northeast-3", "eu-north-1", "sa-east-1", "ca-central-1", "cn-north-1", "cn-northwest-1", "af-south-1", "eu-south-1", "eu-south-2", "ap-southeast-4", "il-central-1", "ca-west-1", "ap-southeast-5", "mx-central-1", "us-isof-south-1", "us-isof-east-1", "ap-southeast-7", "ap-east-2", "eu-isoe-west-1", "ap-southeast-6"] | None = None
    """
    (Private hosted zones only) The region that an Amazon VPC was created in.
    """
    VPCId: str | None = None
    """
    (Private hosted zones only) The ID of an Amazon VPC.
    """

    @property
    def pk(self) -> str | None:
        """
        Return the primary key of the model.   This is the value of the :py:attr:`VPCId`
        attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.VPCId

    def __hash__(self) -> int:
        """
        Return the hash of the model.

        This is the value of the
        :py:attr:`VPCId` attribute.
        """
        return hash(self.VPCId)


class Route53CidrCollection(PrimaryBoto3Model):
    """
    A complex type that identifies a CIDR collection.
    """

    manager_class: ClassVar[Type[Boto3ModelManager]] = Route53CidrCollectionManager

    Arn: str = Field(default=None, frozen=True)
    """
    The ARN of the collection.

    Can be used to reference the collection in IAM policy or in another Amazon Web
    Services account.
    """
    Id: str | None = None
    """
    The unique ID of the CIDR collection.
    """
    Name: str | None = None
    """
    The name of a CIDR collection.
    """
    Version: int = Field(default=None, frozen=True)
    """
    A sequential counter that Route 53 sets to 1 when you create a CIDR collection and
    increments by 1 each time you update settings for the CIDR collection.
    """

    @property
    def pk(self) -> str | None:
        """
        Return the primary key of the model.   This is the value of the :py:attr:`Id`
        attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.Id

    @property
    def arn(self) -> str | None:
        """
        Return the ARN of the model.   This is the value of the :py:attr:`Arn`
        attribute.

        Returns:
            The ARN of the model instance.
        """
        return self.Arn

    @property
    def name(self) -> str | None:
        """
        Return the name of the model.   This is the value of the :py:attr:`Name`
        attribute.

        Returns:
            The name of the model instance.
        """
        return self.Name

    def __hash__(self) -> int:
        """
        Return the hash of the model.

        This is the value of the
        :py:attr:`Id` attribute.
        """
        return hash(self.Id)


class CidrBlockSummary(PrimaryBoto3Model):
    """
    A complex type that lists the CIDR blocks.
    """

    manager_class: ClassVar[Type[Boto3ModelManager]] = CidrBlockSummaryManager

    CidrBlock: str | None = None
    """
    Value for the CIDR block.
    """
    LocationName: str | None = None
    """
    The location name of the CIDR block.
    """

    @property
    def pk(self) -> str | None:
        """
        Return the primary key of the model.   This is the value of the
        :py:attr:`CidrBlock` attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.CidrBlock

    @property
    def name(self) -> str | None:
        """
        Return the name of the model.   This is the value of the :py:attr:`CidrBlock`
        attribute.

        Returns:
            The name of the model instance.
        """
        return self.CidrBlock

    def __hash__(self) -> int:
        """
        Return the hash of the model.

        This is the value of the
        :py:attr:`CidrBlock` attribute.
        """
        return hash(self.CidrBlock)


class Route53QueryLoggingConfig(PrimaryBoto3Model):
    """
    A complex type that contains information about a configuration for DNS query
    logging.
    """

    manager_class: ClassVar[Type[Boto3ModelManager]] = Route53QueryLoggingConfigManager

    Id: str = Field(frozen=True)
    """
    The ID for a configuration for DNS query logging.
    """
    HostedZoneId: str
    """
    The ID of the hosted zone that CloudWatch Logs is logging queries for.
    """
    CloudWatchLogsLogGroupArn: str
    """
    The Amazon Resource Name (ARN) of the CloudWatch Logs log group that Amazon Route 53
    is publishing logs to.
    """

    @property
    def pk(self) -> str | None:
        """
        Return the primary key of the model.   This is the value of the :py:attr:`Id`
        attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.Id

    @property
    def name(self) -> str | None:
        """
        Return the name of the model.   This is the value of the :py:attr:`Id`
        attribute.

        Returns:
            The name of the model instance.
        """
        return self.Id

    def __hash__(self) -> int:
        """
        Return the hash of the model.

        This is the value of the
        :py:attr:`Id` attribute.
        """
        return hash(self.Id)


class Route53GeoLocation(Boto3Model):
    """
    A complex type that contains information about a geographic location.
    """

    ContinentCode: str | None = None
    """
    The two-letter code for the continent.
    """
    CountryCode: str | None = None
    """
    For geolocation resource record sets, the two-letter code for a country.
    """
    SubdivisionCode: str | None = None
    """
    For geolocation resource record sets, the two-letter code for a state of the United
    States.

    Route 53 doesn't support any
    other values for ``SubdivisionCode``. For a list of state abbreviations, see `Appendix B: Two-Letter State and
    Possession Abbreviations <https://pe.usps.com/text/pub28/28apb.htm>`_ on the United States Postal Service website.
    """


class ResourceRecord(Boto3Model):
    """
    Information specific to the resource record.

    If you're creating an alias resource record set, omit ``ResourceRecord``.
    """

    Value: str
    """
    The current or new DNS record value, not to exceed 4,000 characters.

    In the case of a ``DELETE`` action, if the current
    value does not match the actual value, an error is returned. For descriptions about how to format ``Value`` for
    different record types, see `Supported DNS Resource Record
    Types <https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/ResourceRecordTypes.html>`_ in the *Amazon Route 53
    Developer Guide*.
    """


class Route53AliasTarget(Boto3Model):
    """
    *Alias resource record sets only:* Information about the Amazon Web Services
    resource, such as a CloudFront distribution or an Amazon S3 bucket, that you want to
    route traffic to.

    When creating resource record sets for a private hosted zone, note the following:

    * For information about creating failover resource record sets in a private hosted zone, see `Configuring Failover in a
      Private Hosted Zone <https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/dns-failover-private-hosted-zones.html>`_.
    """

    HostedZoneId: str
    """
    *Alias resource records sets only*: The value used depends on where you want to
    route traffic:
    """
    DNSName: str
    """
    *Alias resource record sets only:* The value that you specify depends on where you
    want to route queries:
    """
    EvaluateTargetHealth: bool
    """
    *Applies only to alias, failover alias, geolocation alias, latency alias, and
    weighted alias resource record sets:* When ``EvaluateTargetHealth`` is ``true``, an
    alias resource record set inherits the health of the referenced Amazon Web Services
    resource, such as an ELB load balancer or another resource record set in the hosted
    zone.
    """


class Route53CidrRoutingConfig(Boto3Model):
    """
    The object that is specified in resource record set object when you are linking a
    resource record set to a CIDR location.

    A ``LocationName`` with an asterisk “*” can be used to create a default CIDR record. ``CollectionId`` is still required
    for default record.
    """

    CollectionId: str
    """
    The CIDR collection ID.
    """
    LocationName: str
    """
    The CIDR collection location name.
    """


class Route53Coordinates(Boto3Model):
    """
    A complex type that lists the coordinates for a geoproximity resource record.
    """

    Latitude: str
    """
Specifies a coordinate of the north-south position of a geographic point on the surface of the Earth (-90 - 90).
    """
    Longitude: str
    """
Specifies a coordinate of the east-west position of a geographic point on the surface of the Earth (-180 - 180).
    """


class Route53GeoProximityLocation(Boto3Model):
    """
    (Resource record sets only): A complex type that lets you specify where your
    resources are located. Only one of ``LocalZoneGroup``, ``Coordinates``, or ``Amazon
    Web ServicesRegion`` is allowed per request at a time.

    For more information about geoproximity routing, see
    `Geoproximity routing <https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/routing-policy-geoproximity.html>`_
    in the *Amazon
    Route 53 Developer Guide*.
    """

    AWSRegion: str | None = None
    """
    The Amazon Web Services Region the resource you are directing DNS traffic to, is in.
    """
    LocalZoneGroup: str | None = None
    """
    Specifies an Amazon Web Services Local Zone Group.
    """
    Coordinates: Route53Coordinates | None = None
    """
    Contains the longitude and latitude for a geographic region.
    """
    Bias: int | None = None
    """
    The bias increases or decreases the size of the geographic region from which
    Route 53 routes traffic to a resource.
    """


class Route53ResourceRecordSet(PrimaryBoto3Model):
    """
    Information about the resource record set to create or delete.
    """

    manager_class: ClassVar[Type[Boto3ModelManager]] = Route53ResourceRecordSetManager

    Name: str = Field(frozen=True)
    """
    For ``ChangeResourceRecordSets`` requests, the name of the record that you want to
    create, update, or delete.

    For
    ``ListResourceRecordSets`` responses, the name of a record in the specified hosted zone.
    """
    Type: Literal[
        "SOA",
        "A",
        "TXT",
        "NS",
        "CNAME",
        "MX",
        "NAPTR",
        "PTR",
        "SRV",
        "SPF",
        "AAAA",
        "CAA",
        "DS",
        "TLSA",
        "SSHFP",
        "SVCB",
        "HTTPS",
    ] = Field(frozen=True)
    """
    The DNS record type.

    For information about different record types and how data is encoded for them, see
    `Supported DNS Resource Record Types <https://docs.aws.amazon.com/Route53/latest/DeveloperGuide/ResourceRecordTypes.html>`_
    in the
    *Amazon Route 53 Developer Guide*.
    """
    SetIdentifier: str = Field(default=None, frozen=True)
    """
    *Resource record sets that have a routing policy other than simple:* An identifier
    that differentiates among multiple resource record sets that have the same
    combination of name and type, such as multiple weighted resource record sets named
    acme.example.com that have a type of A.

    In a group of resource record sets that have the same name and type, the
    value of ``SetIdentifier`` must be unique for each resource record set.
    """
    Weight: int = Field(default=None, frozen=True)
    """
    *Weighted resource record sets only:* Among resource record sets that have the same
    combination of DNS name and type, a value that determines the proportion of DNS
    queries that Amazon Route 53 responds to using the current resource record set.

    Route 53 calculates the sum of the weights for the resource record sets that have
    the same combination of DNS name and type. Route 53 then responds to queries based
    on the ratio of a resource's weight to the total. Note the following:
    """
    Region: Literal[
        "us-east-1",
        "us-east-2",
        "us-west-1",
        "us-west-2",
        "ca-central-1",
        "eu-west-1",
        "eu-west-2",
        "eu-west-3",
        "eu-central-1",
        "eu-central-2",
        "ap-southeast-1",
        "ap-southeast-2",
        "ap-southeast-3",
        "ap-northeast-1",
        "ap-northeast-2",
        "ap-northeast-3",
        "eu-north-1",
        "sa-east-1",
        "cn-north-1",
        "cn-northwest-1",
        "ap-east-1",
        "me-south-1",
        "me-central-1",
        "ap-south-1",
        "ap-south-2",
        "af-south-1",
        "eu-south-1",
        "eu-south-2",
        "ap-southeast-4",
        "il-central-1",
        "ca-west-1",
        "ap-southeast-5",
        "mx-central-1",
        "ap-southeast-7",
        "us-gov-east-1",
        "us-gov-west-1",
        "ap-east-2",
        "ap-southeast-6",
    ] = Field(default=None, frozen=True)
    """
    *Latency-based resource record sets only:* The Amazon EC2 Region where you created
    the resource that this resource record set refers to.

    The resource typically is an Amazon Web Services resource, such as an EC2 instance
    or an ELB load balancer, and is referred to by an IP address or a DNS domain name,
    depending on the record type.
    """
    GeoLocation: Route53GeoLocation = Field(default=None, frozen=True)
    """
    *Geolocation resource record sets only:* A complex type that lets you control how
    Amazon Route 53 responds to DNS queries based on the geographic origin of the query.

    For example, if you want all queries from Africa to be routed to a
    web server with an IP address of ``192.0.2.111``, create a resource record set with a ``Type`` of ``A`` and a
    ``ContinentCode`` of ``AF``.
    """
    Failover: Literal["PRIMARY", "SECONDARY"] = Field(default=None, frozen=True)
    """
    *Failover resource record sets only:* To configure failover, you add the
    ``Failover`` element to two resource record sets.

    For one resource record set, you specify ``PRIMARY`` as the value for ``Failover``; for the other resource record
    set, you specify ``SECONDARY``. In addition, you include the ``HealthCheckId`` element and specify the health check that
    you want Amazon Route 53 to perform for each resource record set.
    """
    MultiValueAnswer: bool = Field(default=None, frozen=True)
    """
    *Multivalue answer resource record sets only*: To route traffic approximately
    randomly to multiple resources, such as web servers, create one multivalue answer
    record for each resource and specify ``true`` for ``MultiValueAnswer``.

    Note
    the following:
    """
    TTL: int = Field(default=None, frozen=True)
    """
    The resource record cache time to live (TTL), in seconds.

    Note the following:
    """
    ResourceRecords: List["ResourceRecord"] = Field(default_factory=list, frozen=True)
    """
    Information about the resource records to act upon.
    """
    AliasTarget: Route53AliasTarget = Field(default=None, frozen=True)
    """
    *Alias resource record sets only:* Information about the Amazon Web Services
    resource, such as a CloudFront distribution or an Amazon S3 bucket, that you want to
    route traffic to.
    """
    HealthCheckId: str = Field(default=None, frozen=True)
    """
    If you want Amazon Route 53 to return this resource record set in response to a DNS
    query only when the status of a health check is healthy, include the
    ``HealthCheckId`` element and specify the ID of the applicable health check.
    """
    TrafficPolicyInstanceId: str = Field(default=None, frozen=True)
    """
    When you create a traffic policy instance, Amazon Route 53 automatically creates a
    resource record set.

    ``TrafficPolicyInstanceId`` is the ID of the traffic policy instance that Route 53 created this resource record set for.
    """
    CidrRoutingConfig: Route53CidrRoutingConfig = Field(default=None, frozen=True)
    """
    The object that is specified in resource record set object when you are linking a
    resource record set to a CIDR location.
    """
    GeoProximityLocation: Route53GeoProximityLocation = Field(default=None, frozen=True)
    """
    *GeoproximityLocation resource record sets only:* A complex type that lets you
    control how Route 53 responds to DNS queries based on the geographic origin of the
    query and your resources.
    """

    @property
    def pk(self) -> str | None:
        """
        Return the primary key of the model.   This is the value of the :py:attr:`Name`
        attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.Name

    @property
    def name(self) -> str | None:
        """
        Return the name of the model.   This is the value of the :py:attr:`Name`
        attribute.

        Returns:
            The name of the model instance.
        """
        return self.Name

    def __hash__(self) -> int:
        """
        Return the hash of the model.

        This is the value of the
        :py:attr:`Name` attribute.
        """
        return hash(self.Name)


# =======================
# Request/Response Models
# =======================


class ChangeInfo(Boto3Model):
    """
    A complex type that describes change information about changes made to your hosted
    zone.
    """

    Id: str
    """
This element contains an ID that you use when performing a
`GetChange <https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetChange.html>`_ action to get detailed
information about the change.
    """
    Status: Literal["PENDING", "INSYNC"]
    """
    The current state of the request.

    ``PENDING`` indicates that this request has not yet been applied to all Amazon Route
    53 DNS servers.
    """
    SubmittedAt: datetime
    """
The date and time that the change request was submitted in `ISO 8601 format <https://en.wikipedia.org/wiki/ISO_8601>`_ and
Coordinated Universal Time (UTC). For example, the value ``2017-03-27T17:48:16.751Z`` represents March 27, 2017 at
17:48:16.751 UTC.
    """
    Comment: str | None = None
    """
    A comment you can provide.
    """


class Route53DelegationSet(Boto3Model):
    """
    A complex type that lists the name servers in a delegation set, as well as the
    ``CallerReference`` and the ``ID`` for the delegation set.
    """

    Id: str | None = None
    """
    The ID that Amazon Route 53 assigns to a reusable delegation set.
    """
    CallerReference: str | None = None
    """
    The value that you specified for ``CallerReference`` when you created the reusable
    delegation set.
    """
    NameServers: List[str]
    """
    A complex type that contains a list of the authoritative name servers for a hosted
    zone or for a reusable delegation set.
    """


class CreateHostedZoneResponse(Boto3Model):
    """
    A complex type containing the response information for the hosted zone.
    """

    HostedZone: "HostedZone"
    """
    A complex type that contains general information about the hosted zone.
    """
    ChangeInfo: "ChangeInfo"
    """
    A complex type that contains information about the ``CreateHostedZone`` request.
    """
    DelegationSet: Route53DelegationSet
    """
    A complex type that describes the name servers for this hosted zone.
    """
    VPC: Route53VPC | None = None
    """
    A complex type that contains information about an Amazon VPC that you associated
    with this hosted zone.
    """
    Location: str
    """
    The unique URL representing the new hosted zone.
    """


class DeleteHostedZoneResponse(Boto3Model):
    """
    A complex type that contains the response to a ``DeleteHostedZone`` request.
    """

    ChangeInfo: "ChangeInfo"
    """
    A complex type that contains the ID, the status, and the date and time of a request
    to delete a hosted zone.
    """


class GetHostedZoneResponse(Boto3Model):
    """
    A complex type that contain the response to a ``GetHostedZone`` request.
    """

    HostedZone: "HostedZone"
    """
    A complex type that contains general information about the specified hosted zone.
    """
    DelegationSet: Route53DelegationSet | None = None
    """
    A complex type that lists the Amazon Route 53 name servers for the specified hosted
    zone.
    """
    VPCs: List["Route53VPC"] | None = None
    """
    A complex type that contains information about the VPCs that are associated with the
    specified hosted zone.
    """


class ListHostedZonesResponse(Boto3Model):
    Marker: str | None = None
    """
    For the second and subsequent calls to ``ListHostedZones``, ``Marker`` is the value
    that you specified for the ``marker`` parameter in the request that produced the
    current response.
    """
    HostedZones: List["HostedZone"]
    """
    A complex type that contains general information about the hosted zone.
    """
    IsTruncated: bool
    """
    A flag indicating whether there are more hosted zones to be listed.

    If the response was truncated, you can get more
    hosted zones by submitting another ``ListHostedZones`` request and specifying the value of ``NextMarker`` in the
    ``marker`` parameter.
    """
    NextMarker: str | None = None
    """
    If ``IsTruncated`` is ``true``, the value of ``NextMarker`` identifies the first
    hosted zone in the next group of hosted zones.

    Submit another ``ListHostedZones`` request, and specify the value of ``NextMarker`` from the response in the
    ``marker`` parameter.
    """
    MaxItems: str
    """
    The value that you specified for the ``maxitems`` parameter in the call to
    ``ListHostedZones`` that produced the current response.
    """


class ListResourceRecordSetsResponse(Boto3Model):
    """
    A complex type that contains list information for the resource record set.
    """

    ResourceRecordSets: List["Route53ResourceRecordSet"]
    """
    Information about multiple resource record sets.
    """
    IsTruncated: bool
    """
    A flag that indicates whether more resource record sets remain to be listed.

    If your results were truncated, you can
    make a follow-up pagination request by using the ``NextRecordName`` element.
    """
    NextRecordName: str | None = None
    """
    If the results were truncated, the name of the next record in the list.
    """
    NextRecordType: Literal["SOA", "A", "TXT", "NS", "CNAME", "MX", "NAPTR", "PTR", "SRV", "SPF", "AAAA", "CAA", "DS", "TLSA", "SSHFP", "SVCB", "HTTPS"] | None = None
    """
    If the results were truncated, the type of the next record in the list.
    """
    NextRecordIdentifier: str | None = None
    """
    *Resource record sets that have a routing policy other than simple:* If results were
    truncated for a given DNS name and type, the value of ``SetIdentifier`` for the next
    resource record set that has the current DNS name and type.
    """
    MaxItems: str
    """
    The maximum number of records you requested.
    """


class ListHostedZonesByNameResponse(Boto3Model):
    """
    A complex type that contains the response information for the request.
    """

    HostedZones: List["HostedZone"]
    """
    A complex type that contains general information about the hosted zone.
    """
    DNSName: str | None = None
    """
    For the second and subsequent calls to ``ListHostedZonesByName``, ``DNSName`` is the
    value that you specified for the ``dnsname`` parameter in the request that produced
    the current response.
    """
    HostedZoneId: str | None = None
    """
    The ID that Amazon Route 53 assigned to the hosted zone when you created it.
    """
    IsTruncated: bool
    """
    A flag that indicates whether there are more hosted zones to be listed.

    If the response was truncated, you can get the
    next group of ``maxitems`` hosted zones by calling ``ListHostedZonesByName`` again and specifying the values of
    ``NextDNSName`` and ``NextHostedZoneId`` elements in the ``dnsname`` and ``hostedzoneid`` parameters.
    """
    NextDNSName: str | None = None
    """
    If ``IsTruncated`` is true, the value of ``NextDNSName`` is the name of the first
    hosted zone in the next group of ``maxitems`` hosted zones.

    Call ``ListHostedZonesByName`` again and specify the value of ``NextDNSName`` and
    ``NextHostedZoneId`` in the ``dnsname`` and ``hostedzoneid`` parameters, respectively.
    """
    NextHostedZoneId: str | None = None
    """
    If ``IsTruncated`` is ``true``, the value of ``NextHostedZoneId`` identifies the
    first hosted zone in the next group of ``maxitems`` hosted zones.

    Call ``ListHostedZonesByName`` again and specify the value of ``NextDNSName`` and
    ``NextHostedZoneId`` in the ``dnsname`` and ``hostedzoneid`` parameters, respectively.
    """
    MaxItems: str
    """
    The value that you specified for the ``maxitems`` parameter in the call to
    ``ListHostedZonesByName`` that produced the current response.
    """


class UpdateHostedZoneCommentResponse(Boto3Model):
    """
    A complex type that contains the response to the ``UpdateHostedZoneComment``
    request.
    """

    HostedZone: "HostedZone"
    """
    A complex type that contains the response to the ``UpdateHostedZoneComment``
    request.
    """


class GetHostedZoneCountResponse(Boto3Model):
    """
    A complex type that contains the response to a ``GetHostedZoneCount`` request.
    """

    HostedZoneCount: int
    """
    The total number of public and private hosted zones that are associated with the
    current Amazon Web Services account.
    """


class HostedZoneLimit(Boto3Model):
    """
    A complex type that contains the type of limit that you specified in the request and
    the current value for that limit.
    """

    Type: Literal["MAX_RRSETS_BY_ZONE", "MAX_VPCS_ASSOCIATED_BY_ZONE"]
    """
    The limit that you requested.

    Valid values include the following:
    """
    Value: int
    """
    The current value for the limit that is specified by ``Type``.
    """


class GetHostedZoneLimitResponse(Boto3Model):
    """
    A complex type that contains the requested limit.
    """

    Limit: HostedZoneLimit
    """
    The current setting for the specified limit.

    For example, if you specified ``MAX_RRSETS_BY_ZONE`` for the value of
    ``Type`` in the request, the value of ``Limit`` is the maximum number of records that you can create in the specified
    hosted zone.
    """
    Count: int
    """
    The current number of entities that you have created of the specified type.

    For example, if you specified
    ``MAX_RRSETS_BY_ZONE`` for the value of ``Type`` in the request, the value of ``Count`` is the current number of records
    that you have created in the specified hosted zone.
    """


class AssociateVPCWithHostedZoneResponse(Boto3Model):
    """
    A complex type that contains the response information for the
    ``AssociateVPCWithHostedZone`` request.
    """

    ChangeInfo: "ChangeInfo"
    """
    A complex type that describes the changes made to your hosted zone.
    """


class DisassociateVPCFromHostedZoneResponse(Boto3Model):
    """
    A complex type that contains the response information for the disassociate request.
    """

    ChangeInfo: "ChangeInfo"
    """
    A complex type that describes the changes made to the specified private hosted zone.
    """


class HostedZoneOwner(Boto3Model):
    """
    A complex type that identifies a hosted zone that a specified Amazon VPC is
    associated with and the owner of the hosted zone.

    If there is a value for ``OwningAccount``, there is no value for ``OwningService``, and vice versa.
    """

    OwningAccount: str | None = None
    """
    If the hosted zone was created by an Amazon Web Services account, or was created by
    an Amazon Web Services service that creates hosted zones using the current account,
    ``OwningAccount`` contains the account ID of that account.

    For example, when you use Cloud Map to create a hosted zone, Cloud Map creates the
    hosted zone using the current Amazon Web Services account.
    """
    OwningService: str | None = None
    """
    If an Amazon Web Services service uses its own account to create a hosted zone and
    associate the specified VPC with that hosted zone, ``OwningService`` contains an
    abbreviation that identifies the service.

    For example, if Amazon Elastic File
    System (Amazon EFS) created a hosted zone and associated a VPC with the hosted zone, the value of ``OwningService`` is
    ``efs.amazonaws.com``.
    """


class HostedZoneSummary(Boto3Model):
    """
    In the response to a ``ListHostedZonesByVPC`` request, the ``HostedZoneSummaries``
    element contains one ``HostedZoneSummary`` element for each hosted zone that the
    specified Amazon VPC is associated with.

    Each
    ``HostedZoneSummary`` element contains the hosted zone name and ID, and information about who owns the hosted zone.
    """

    HostedZoneId: str
    """
    The Route 53 hosted zone ID of a private hosted zone that the specified VPC is
    associated with.
    """
    Name: str
    """
    The name of the private hosted zone, such as ``example.com``.
    """
    Owner: HostedZoneOwner
    """
    The owner of a private hosted zone that the specified VPC is associated with.

    The owner can be either an Amazon Web Services account or an Amazon Web Services
    service.
    """


class ListHostedZonesByVPCResponse(Boto3Model):
    HostedZoneSummaries: List["HostedZoneSummary"]
    """
    A list that contains one ``HostedZoneSummary`` element for each hosted zone that the
    specified Amazon VPC is associated with.

    Each ``HostedZoneSummary`` element contains the hosted zone name and ID, and information about who owns the hosted
    zone.
    """
    MaxItems: str
    """
    The value that you specified for ``MaxItems`` in the most recent
    ``ListHostedZonesByVPC`` request.
    """
    NextToken: str | None = None
    """
    The value that you will use for ``NextToken`` in the next ``ListHostedZonesByVPC``
    request.
    """


class ListVPCAssociationAuthorizationsResponse(Boto3Model):
    """
    A complex type that contains the response information for the request.
    """

    HostedZoneId: str
    """
    The ID of the hosted zone that you can associate the listed VPCs with.
    """
    NextToken: str | None = None
    """
    When the response includes a ``NextToken`` element, there are more VPCs that can be
    associated with the specified hosted zone.

    To get the next page of VPCs, submit another ``ListVPCAssociationAuthorizations`` request, and include the value
    of the ``NextToken`` element from the response in the ``nexttoken`` request parameter.
    """
    VPCs: List["Route53VPC"]
    """
    The list of VPCs that are authorized to be associated with the specified hosted
    zone.
    """


class CreateVPCAssociationAuthorizationResponse(Boto3Model):
    """
    A complex type that contains the response information from a
    ``CreateVPCAssociationAuthorization`` request.
    """

    HostedZoneId: str
    """
    The ID of the hosted zone that you authorized associating a VPC with.
    """
    VPC: Route53VPC
    """
    The VPC that you authorized associating with a hosted zone.
    """


class DeleteVPCAssociationAuthorizationResponse(Boto3Model):
    """
    Empty response for the request.
    """



class CreateCidrCollectionResponse(Boto3Model):
    Collection: Route53CidrCollection | None = None
    """
    A complex type that contains information about the CIDR collection.
    """
    Location: str | None = None
    """
    A unique URL that represents the location for the CIDR collection.
    """


class DeleteCidrCollectionResponse(Boto3Model):
    pass


class CollectionSummary(Boto3Model):
    """A complex type that is an entry in an
    `CidrCollection <https://docs.aws.amazon.com/Route53/latest/APIReference/API_CidrCollection.html>`_ array.
    """

    Arn: str | None = None
    """
    The ARN of the collection summary.

    Can be used to reference the collection in IAM policy or cross-account.
    """
    Id: str | None = None
    """
    Unique ID for the CIDR collection.
    """
    Name: str | None = None
    """
    The name of a CIDR collection.
    """
    Version: int | None = None
    """
    A sequential counter that Route 53 sets to 1 when you create a CIDR collection and
    increments by 1 each time you update settings for the CIDR collection.
    """


class ListCidrCollectionsResponse(Boto3Model):
    NextToken: str | None = None
    """
    An opaque pagination token to indicate where the service is to begin enumerating
    results.
    """
    CidrCollections: List["CollectionSummary"] | None = None
    """
    A complex type with information about the CIDR collection.
    """


class CidrCollectionChange(Boto3Model):
    """
    A complex type that contains information about the CIDR collection change.
    """

    LocationName: str
    """
    Name of the location that is associated with the CIDR collection.
    """
    Action: Literal["PUT", "DELETE_IF_EXISTS"]
    """
    CIDR collection change action.
    """
    CidrList: List[str]
    """
    List of CIDR blocks.
    """


class ChangeCidrCollectionResponse(Boto3Model):
    Id: str
    """
    The ID that is returned by ``ChangeCidrCollection``.

    You can use it as input to ``GetChange`` to see if a CIDR
    collection change has propagated or not.
    """


class ListCidrBlocksResponse(Boto3Model):
    NextToken: str | None = None
    """
    An opaque pagination token to indicate where the service is to begin enumerating
    results.
    """
    CidrBlocks: List["CidrBlockSummary"] | None = None
    """
    A complex type that contains information about the CIDR blocks.
    """


class CreateQueryLoggingConfigResponse(Boto3Model):
    QueryLoggingConfig: Route53QueryLoggingConfig
    """
    A complex type that contains the ID for a query logging configuration, the ID of the
    hosted zone that you want to log queries for, and the ARN for the log group that you
    want Amazon Route 53 to send query logs to.
    """
    Location: str
    """
    The unique URL representing the new query logging configuration.
    """


class DeleteQueryLoggingConfigResponse(Boto3Model):
    pass


class GetQueryLoggingConfigResponse(Boto3Model):
    QueryLoggingConfig: Route53QueryLoggingConfig
    """
A complex type that contains information about the query logging configuration that you specified in a
`GetQueryLoggingConfig <https://docs.aws.amazon.com/Route53/latest/APIReference/API_GetQueryLoggingConfig.html>`_ request.
    """


class ListQueryLoggingConfigsResponse(Boto3Model):
    QueryLoggingConfigs: List["Route53QueryLoggingConfig"]
    """
An array that contains one
`QueryLoggingConfig <https://docs.aws.amazon.com/Route53/latest/APIReference/API_QueryLoggingConfig.html>`_ element for
each configuration for DNS query logging that is associated with the current Amazon Web Services account.
    """
    NextToken: str | None = None
    """
    If a response includes the last of the query logging configurations that are
    associated with the current Amazon Web Services account, ``NextToken`` doesn't
    appear in the response.
    """


class Change(Boto3Model):
    """
    The information for each resource record set that you want to change.
    """

    Action: Literal["CREATE", "DELETE", "UPSERT"]
    """
The action to perform:
    """
    ResourceRecordSet: Route53ResourceRecordSet
    """
    Information about the resource record set to create, delete, or update.
    """


class ChangeBatch(Boto3Model):
    """
    The information for a change request.
    """

    Comment: str | None = None
    """
    *Optional:* Any comments you want to include about a change batch request.
    """
    Changes: List["Change"]
    """
    Information about the changes to make to the record sets.
    """


class ChangeResourceRecordSetsResponse(Boto3Model):
    """
    A complex type containing the response for the request.
    """

    ChangeInfo: "ChangeInfo"
    """
    A complex type that contains information about changes made to your hosted zone.
    """



from .manager.base import ManagerMethodGenerator  # noqa: F401
from .manager.create import CreateMethodGenerator  # noqa: F401
from .manager.delete import DeleteMethodGenerator  # noqa: F401
from .manager.general import GeneralMethodGenerator  # noqa: F401
from .manager.get import GetMethodGenerator  # noqa: F401
from .manager.get_many import GetManyMethodGenerator  # noqa: F401
from .manager.list import ListMethodGenerator  # noqa: F401
from .manager.partial_update import PartialUpdateMethodGenerator  # noqa: F401
from .manager.update import UpdateMethodGenerator  # noqa: F401
from .model.methods import ModelManagerMethodGenerator  # noqa: F401
from .model.properties import ModelPropertyGenerator  # noqa: F401
from .model.relations import ModelRelationGenerator  # noqa: F401



# This file is automatically generated by botocraft.  Do not edit directly.
# mypy: disable-error-code="index, override, assignment, union-attr, misc"
from botocraft.mixins.tags import TagsDictMixin
from .abstract import PrimaryBoto3ModelQuerySet
from typing import ClassVar, Type, Optional, Literal, Dict, List, Union, Any, cast
from datetime import datetime
from botocraft.services.common import Tag
from pydantic import Field
from .abstract import (
    Boto3Model,
    ReadonlyBoto3Model,
    PrimaryBoto3Model,
    ReadonlyPrimaryBoto3Model,
    Boto3ModelManager,
    ReadonlyBoto3ModelManager,
)
from botocraft.mixins.kms import kms_keys_only

# ===============
# Managers
# ===============


class KMSKeyManager(Boto3ModelManager):
    service_name: str = "kms"

    def create(
        self,
        model: "KMSKey",
        Policy: str | None = None,
        BypassPolicyLockoutSafetyCheck: bool | None = None,
        Tags: List[Tag] | None = None,
        XksKeyId: str | None = None,
    ) -> "KMSKey":
        """
        Creates a unique customer managed `KMS key <https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#kms-
        keys>`_ in your Amazon Web Services account and Region. You can use a KMS key in cryptographic operations, such as
        encryption and signing. Some Amazon Web Services services let you use KMS keys that you create and manage to protect
        your service resources.

        Args:
            model: The :py:class:`KeyMetadata` to create.

        Keyword Args:
            Policy: The key policy to attach to the KMS key.
            BypassPolicyLockoutSafetyCheck: Skips ("bypasses") the key policy lockout safety check. The default value is false.
            Tags: Assigns one or more tags to the KMS key. Use this parameter to tag the KMS key when it is created. To tag an
                existing KMS key, use the TagResource operation.
            XksKeyId: Identifies the `external key <https://docs.aws.amazon.com/kms/latest/developerguide/keystore-
                external.html#concept- external-key>`_ that serves as key material for the KMS key in an `external key store
                <https://docs.aws.amazon.com/kms/latest/developerguide/keystore-external.html>`_. Specify the ID that the `external
                key store proxy <https://docs.aws.amazon.com/kms/latest/developerguide/keystore-external.html#concept-xks-proxy>`_
                uses to refer to the external key. For help, see the documentation for your external key store proxy.

        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            Policy=self.serialize(Policy),
            Description=data.get("Description"),
            KeyUsage=data.get("KeyUsage"),
            CustomerMasterKeySpec=data.get("CustomerMasterKeySpec"),
            KeySpec=data.get("KeySpec"),
            Origin=data.get("Origin"),
            CustomKeyStoreId=data.get("CustomKeyStoreId"),
            BypassPolicyLockoutSafetyCheck=self.serialize(
                BypassPolicyLockoutSafetyCheck
            ),
            Tags=self.serialize(Tags),
            MultiRegion=data.get("MultiRegion"),
            XksKeyId=self.serialize(XksKeyId),
        )
        _response = self.client.create_key(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateKeyResponse(**_response)

        self.sessionize(response.KeyMetadata)
        return cast("KMSKey", response.KeyMetadata)

    def get(
        self, KeyId: str, *, GrantTokens: List[str] | None = None
    ) -> Optional["KMSKey"]:
        """
        Provides detailed information about a KMS key. You can run ``DescribeKey`` on a `customer managed
        key <https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#customer-mgn-key>`_ or an `Amazon Web Services
        managed key <https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#aws-managed-key>`_.

        Args:
            KeyId: Describes the specified KMS key.

        Keyword Args:
            GrantTokens: A list of grant tokens.

        """
        args: Dict[str, Any] = dict(
            KeyId=self.serialize(KeyId), GrantTokens=self.serialize(GrantTokens)
        )
        _response = self.client.describe_key(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DescribeKeyResponse(**_response)

        if response and response.KeyMetadata:
            self.sessionize(response.KeyMetadata)
            return response.KeyMetadata
        return None

    @kms_keys_only
    def list(self, *, Limit: int | None = None) -> PrimaryBoto3ModelQuerySet:
        """
        Gets a list of all KMS keys in the caller's Amazon Web Services account and
        Region.

        Keyword Args:
            Limit: Use this parameter to specify the maximum number of items to return. When this value is present, KMS does not
                return more than the specified number of items, but it might return fewer.
        """
        paginator = self.client.get_paginator("list_keys")
        args: Dict[str, Any] = dict(Limit=self.serialize(Limit))
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results = []
        for _response in response_iterator:
            if list(_response.keys()) == ["ResponseMetadata"]:
                break
            if "ResponseMetadata" in _response:
                del _response["ResponseMetadata"]
            response = ListKeysResponse(**_response)
            if response.Keys:
                results.extend(response.Keys)
            else:
                if getattr(response, "NextToken", None):
                    continue
                break
        self.sessionize(results)
        if results and isinstance(results[0], Boto3Model):
            return PrimaryBoto3ModelQuerySet(results)
        return results

    def delete(
        self, KeyId: str, *, PendingWindowInDays: int = 7
    ) -> "ScheduleKeyDeletionResponse":
        """
        Schedules the deletion of a KMS key. By default, KMS applies a waiting period of
        30 days, but you can specify a waiting period of 7-30 days. When this operation
        is successful, the key state of the KMS key changes to ``PendingDeletion`` and
        the key can't be used in any cryptographic operations. It remains in this state
        for the duration of the waiting period. Before the waiting period ends, you can
        use CancelKeyDeletion to cancel the deletion of the KMS key. After the waiting
        period ends, KMS deletes the KMS key, its key material, and all KMS data
        associated with it, including all aliases that refer to it.

        Args:
            KeyId: The unique identifier of the KMS key to delete.

        Keyword Args:
            PendingWindowInDays: The waiting period, specified in number of days. After the waiting period ends, KMS deletes the
                KMS key.
        """
        args: Dict[str, Any] = dict(
            KeyId=self.serialize(KeyId),
            PendingWindowInDays=self.serialize(PendingWindowInDays),
        )
        _response = self.client.schedule_key_deletion(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ScheduleKeyDeletionResponse(**_response)
        return response

    def enable(self, KeyId: str) -> None:
        """
        Sets the key state of a KMS key to enabled. This allows you to use the KMS key for `cryptographic
        operations <https://docs.aws.amazon.com/kms/latest/developerguide/kms-cryptography.html#cryptographic-operations>`_.

        Args:
            KeyId: Identifies the KMS key to enable.

        """
        args: Dict[str, Any] = dict(KeyId=self.serialize(KeyId))
        self.client.enable_key(**{k: v for k, v in args.items() if v is not None})

    def disable(self, KeyId: str) -> None:
        """
        Sets the state of a KMS key to disabled. This change temporarily prevents use of the KMS key for `cryptographic
        operations <https://docs.aws.amazon.com/kms/latest/developerguide/kms-cryptography.html#cryptographic-operations>`_.

        Args:
            KeyId: Identifies the KMS key to disable.

        """
        args: Dict[str, Any] = dict(KeyId=self.serialize(KeyId))
        self.client.disable_key(**{k: v for k, v in args.items() if v is not None})

    def cancel_deletion(self, KeyId: str) -> str:
        """
        Cancels the deletion of a KMS key. When this operation succeeds, the key state
        of the KMS key is ``Disabled``. To enable the KMS key, use EnableKey.

        Args:
            KeyId: Identifies the KMS key whose deletion is being canceled.
        """
        args: Dict[str, Any] = dict(KeyId=self.serialize(KeyId))
        _response = self.client.cancel_key_deletion(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CancelKeyDeletionResponse(**_response)

        results: str = None
        if response is not None:
            results = response.KeyId

        self.sessionize(results)
        return cast("str", results)

    def create_alias(self, AliasName: str, TargetKeyId: str) -> "None":
        """
        Creates a friendly name for a KMS key.

        Args:
            AliasName: Specifies the alias name. This value must begin with ``alias/`` followed by a name, such as
                ``alias/ExampleAlias``.
            TargetKeyId: Associates the alias with the specified `customer managed key
                <https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#customer-mgn-key>`_. The KMS key must be in the
                same Amazon Web Services Region.
        """
        args: Dict[str, Any] = dict(
            AliasName=self.serialize(AliasName), TargetKeyId=self.serialize(TargetKeyId)
        )
        self.client.create_alias(**{k: v for k, v in args.items() if v is not None})

    def update_alias(self, AliasName: str, TargetKeyId: str) -> "None":
        """
        Associates an existing KMS alias with a different KMS key. Each alias is
        associated with only one KMS key at a time, although a KMS key can have multiple
        aliases. The alias and the KMS key must be in the same Amazon Web Services
        account and Region.

        Args:
            AliasName: Identifies the alias that is changing its KMS key. This value must begin with ``alias/`` followed by the
                alias name, such as ``alias/ExampleAlias``. You cannot use ``UpdateAlias`` to change the alias name.
            TargetKeyId: Identifies the `customer managed key
                <https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#customer-mgn- key>`_ to associate with the
                alias. You don't have permission to associate an alias with an `Amazon Web Services managed key
                <https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#aws-managed-key>`_.
        """
        args: Dict[str, Any] = dict(
            AliasName=self.serialize(AliasName), TargetKeyId=self.serialize(TargetKeyId)
        )
        self.client.update_alias(**{k: v for k, v in args.items() if v is not None})


# ==============
# Service Models
# ==============


class MultiRegionKey(Boto3Model):
    """
    Describes the primary or replica key in a multi-Region key.
    """

    Arn: str | None = None
    """
    Displays the key ARN of a primary or replica key of a multi-Region key.
    """
    Region: str | None = None
    """
    Displays the Amazon Web Services Region of a primary or replica key in a multi-
    Region key.
    """


class KMSMultiRegionConfiguration(Boto3Model):
    """
    Describes the configuration of this multi-Region key. This field appears only when
    the KMS key is a primary or replica of a multi-Region key.

    For more information about any listed KMS key, use the DescribeKey operation.
    """

    MultiRegionKeyType: Literal["PRIMARY", "REPLICA"] | None = None
    """
    Indicates whether the KMS key is a ``PRIMARY`` or ``REPLICA`` key.
    """
    PrimaryKey: MultiRegionKey | None = None
    """
    Displays the key ARN and Region of the primary key.

    This field includes the current KMS key if it is the primary key.
    """
    ReplicaKeys: List["MultiRegionKey"] | None = None
    """
    Displays the key ARNs and Regions of all replica keys.

    This field includes the current KMS key if it is a replica key.
    """


class XksKeyConfigurationType(Boto3Model):
    """Information about the `external key <https://docs.aws.amazon.com/kms/latest/developerguide/keystore-
    external.html#concept-external-key>`_ that is associated with a KMS key in an external key store.

    This element appears in a CreateKey or DescribeKey response only for a KMS key in an external key store.

    The *external key* is a symmetric encryption key that is hosted by an external key manager outside of Amazon Web
    Services. When you use the KMS key in an external key store in a cryptographic operation, the cryptographic operation is
    performed in the external key manager using the specified external key. For more information, see `External
    key <https://docs.aws.amazon.com/kms/latest/developerguide/keystore-external.html#concept-external-key>`_ in the *Key
    Management Service Developer Guide*.
    """

    Id: str | None = None
    """
    The ID of the external key in its external key manager.

    This is the ID that the external key store proxy uses to identify the external key.
    """


class KMSKey(PrimaryBoto3Model):
    """
    Contains metadata about a KMS key.

    This data type is used as a response element for the CreateKey, DescribeKey, and
    ReplicateKey operations.
    """

    manager_class: ClassVar[Type[Boto3ModelManager]] = KMSKeyManager

    Enabled: bool | None = True
    """
    Specifies whether the KMS key is enabled.

    When ``KeyState`` is ``Enabled`` this value is true, otherwise it is false.
    """
    KeyUsage: Literal["SIGN_VERIFY", "ENCRYPT_DECRYPT", "GENERATE_VERIFY_MAC", "KEY_AGREEMENT"] | None = "ENCRYPT_DECRYPT"
    """
The `cryptographic operations <https://docs.aws.amazon.com/kms/latest/developerguide/kms-
cryptography.html#cryptographic-operations>`_ for which you can use the KMS key.
    """
    KeySpec: Literal["RSA_2048", "RSA_3072", "RSA_4096", "ECC_NIST_P256", "ECC_NIST_P384", "ECC_NIST_P521", "ECC_SECG_P256K1", "SYMMETRIC_DEFAULT", "HMAC_224", "HMAC_256", "HMAC_384", "HMAC_512", "SM2", "ML_DSA_44", "ML_DSA_65", "ML_DSA_87"] | None = "SYMMETRIC_DEFAULT"
    """
    Describes the type of key material in the KMS key.
    """
    AWSAccountId: str = Field(default=None, frozen=True)
    """
    The twelve-digit account ID of the Amazon Web Services account that owns the KMS
    key.
    """
    KeyId: str = Field(frozen=True)
    """
    The globally unique identifier for the KMS key.
    """
    Arn: str = Field(default=None, frozen=True)
    """
    The Amazon Resource Name (ARN) of the KMS key.

    For examples, see `Key Management Service
    (KMS) <https://docs.aws.amazon.com/general/latest/gr/aws-arns-and-namespaces.html#arn-syntax-kms>`_ in the Example ARNs
    section of the *Amazon Web Services General Reference*.
    """
    CreationDate: datetime = Field(default=None, frozen=True)
    """
    The date and time when the KMS key was created.
    """
    Description: str | None = None
    """
    The description of the KMS key.
    """
    KeyState: Literal[
        "Creating",
        "Enabled",
        "Disabled",
        "PendingDeletion",
        "PendingImport",
        "PendingReplicaDeletion",
        "Unavailable",
        "Updating",
    ] = Field(default=None, frozen=True)
    """
    The current status of the KMS key.
    """
    DeletionDate: datetime = Field(default=None, frozen=True)
    """
    The date and time after which KMS deletes this KMS key.

    This value is present only when the KMS key is scheduled for
    deletion, that is, when its ``KeyState`` is ``PendingDeletion``.
    """
    ValidTo: datetime = Field(default=None, frozen=True)
    """
    The earliest time at which any imported key material permanently associated with
    this KMS key expires.

    When a key
    material expires, KMS deletes the key material and the KMS key becomes unusable. This value is present only for KMS keys
    whose ``Origin`` is ``EXTERNAL`` and the ``ExpirationModel`` is ``KEY_MATERIAL_EXPIRES``, otherwise this value is
    omitted.
    """
    Origin: Literal["AWS_KMS", "EXTERNAL", "AWS_CLOUDHSM", "EXTERNAL_KEY_STORE"] | None = None
    """
    The source of the key material for the KMS key.

    When this value is ``AWS_KMS``, KMS created the key material. When this
    value is ``EXTERNAL``, the key material was imported or the KMS key doesn't have any key material. When this value is
    ``AWS_CLOUDHSM``, the key material was created in the CloudHSM cluster associated with a custom key store.
    """
    CustomKeyStoreId: str | None = None
    """
A unique identifier for the `custom key store <https://docs.aws.amazon.com/kms/latest/developerguide/key-store-
overview.html>`_ that contains the KMS key. This field is present only when the KMS key is created in a custom key store.
    """
    CloudHsmClusterId: str = Field(default=None, frozen=True)
    """
    The cluster ID of the CloudHSM cluster that contains the key material for the KMS
    key.

    When you create a KMS key in an CloudHSM
    `custom key store <https://docs.aws.amazon.com/kms/latest/developerguide/key-store-overview.html>`_,
    KMS creates
    the key material for the KMS key in the associated CloudHSM cluster. This field is present only when the KMS key is
    created in an CloudHSM key store.
    """
    ExpirationModel: Literal["KEY_MATERIAL_EXPIRES", "KEY_MATERIAL_DOES_NOT_EXPIRE"] = (
        Field(default=None, frozen=True)
    )
    """
    Specifies whether the KMS key's key material expires.

    This value is present only when ``Origin`` is ``EXTERNAL``,
    otherwise this value is omitted.
    """
    KeyManager: Literal["AWS", "CUSTOMER"] = Field(default=None, frozen=True)
    """
    The manager of the KMS key.

    KMS keys in your Amazon Web Services account are either customer managed or Amazon
    Web Services managed. For more information about the difference, see
    `KMS keys <https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#kms_keys>`_
    in the *Key Management Service
    Developer Guide*.
    """
    CustomerMasterKeySpec: Literal["RSA_2048", "RSA_3072", "RSA_4096", "ECC_NIST_P256", "ECC_NIST_P384", "ECC_NIST_P521", "ECC_SECG_P256K1", "SYMMETRIC_DEFAULT", "HMAC_224", "HMAC_256", "HMAC_384", "HMAC_512", "SM2"] | None = None
    """
    Instead, use the ``KeySpec`` field.
    """
    EncryptionAlgorithms: List[
        Literal["SYMMETRIC_DEFAULT", "RSAES_OAEP_SHA_1", "RSAES_OAEP_SHA_256", "SM2PKE"]
    ] = Field(default_factory=list, frozen=True)
    """
    The encryption algorithms that the KMS key supports.

    You cannot use the KMS key with other encryption algorithms within KMS.
    """
    SigningAlgorithms: List[
        Literal[
            "RSASSA_PSS_SHA_256",
            "RSASSA_PSS_SHA_384",
            "RSASSA_PSS_SHA_512",
            "RSASSA_PKCS1_V1_5_SHA_256",
            "RSASSA_PKCS1_V1_5_SHA_384",
            "RSASSA_PKCS1_V1_5_SHA_512",
            "ECDSA_SHA_256",
            "ECDSA_SHA_384",
            "ECDSA_SHA_512",
            "SM2DSA",
            "ML_DSA_SHAKE_256",
        ]
    ] = Field(default_factory=list, frozen=True)
    """
    The signing algorithms that the KMS key supports.

    You cannot use the KMS key with other signing algorithms within KMS.
    """
    KeyAgreementAlgorithms: List[Literal["ECDH"]] = Field(
        default_factory=list, frozen=True
    )
    """
    The key agreement algorithm used to derive a shared secret.
    """
    MultiRegion: bool | None = None
    """
    Indicates whether the KMS key is a multi-Region (``True``) or regional (``False``)
    key.

    This value is ``True`` for
    multi-Region primary and replica keys and ``False`` for regional KMS keys.
    """
    MultiRegionConfiguration: KMSMultiRegionConfiguration = Field(
        default=None, frozen=True
    )
    """
    Lists the primary and replica keys in same multi-Region key.

    This field is present only when the value of the
    ``MultiRegion`` field is ``True``.
    """
    PendingDeletionWindowInDays: int = Field(default=None, frozen=True)
    """
    The waiting period before the primary key in a multi-Region key is deleted.

    This waiting period begins when the last of
    its replica keys is deleted. This value is present only when the ``KeyState`` of the KMS key is
    ``PendingReplicaDeletion``. That indicates that the KMS key is the primary key in a multi-Region key, it is scheduled
    for deletion, and it still has existing replica keys.
    """
    MacAlgorithms: List[
        Literal["HMAC_SHA_224", "HMAC_SHA_256", "HMAC_SHA_384", "HMAC_SHA_512"]
    ] = Field(default_factory=list, frozen=True)
    """
    The message authentication code (MAC) algorithm that the HMAC KMS key supports.
    """
    XksKeyConfiguration: XksKeyConfigurationType = Field(default=None, frozen=True)
    """
    Information about the external key that is associated with a KMS key in an external
    key store.
    """
    CurrentKeyMaterialId: str = Field(default=None, frozen=True)
    """
    Identifies the current key material.

    This value is present for symmetric encryption keys with ``AWS_KMS`` origin and
    single-Region, symmetric encryption keys with ``EXTERNAL`` origin. These KMS keys support automatic or on-demand key
    rotation and can have multiple key materials associated with them. KMS uses the current key material for both encryption
    and decryption, and the non-current key material for decryption operations only.
    """

    @property
    def pk(self) -> str | None:
        """
        Return the primary key of the model.   This is the value of the :py:attr:`KeyId`
        attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.KeyId

    @property
    def arn(self) -> str | None:
        """
        Return the ARN of the model.   This is the value of the :py:attr:`Arn`
        attribute.

        Returns:
            The ARN of the model instance.
        """
        return self.Arn

    def __hash__(self) -> int:
        """
        Return the hash of the model.

        This is the value of the
        :py:attr:`KeyId` attribute.
        """
        return hash(self.KeyId)


# =======================
# Request/Response Models
# =======================


class CreateKeyResponse(Boto3Model):
    KeyMetadata: KMSKey | None = None
    """
    Metadata associated with the KMS key.
    """


class DescribeKeyResponse(Boto3Model):
    KeyMetadata: KMSKey | None = None
    """
    Metadata associated with the key.
    """


class KeyListEntry(Boto3Model):
    """
    Contains information about each entry in the key list.
    """

    KeyId: str | None = None
    """
    Unique identifier of the key.
    """
    KeyArn: str | None = None
    """
    ARN of the key.
    """


class ListKeysResponse(Boto3Model):
    Keys: List["KeyListEntry"] | None = None
    """
    A list of KMS keys.
    """
    NextMarker: str | None = None
    """
    When ``Truncated`` is true, this element is present and contains the value to use
    for the ``Marker`` parameter in a subsequent request.
    """
    Truncated: bool | None = None
    """
    A flag that indicates whether there are more items in the list.

    When this value is true, the list in this response is
    truncated. To get more items, pass the value of the ``NextMarker`` element in this response to the ``Marker`` parameter
    in a subsequent request.
    """


class ScheduleKeyDeletionResponse(Boto3Model):
    KeyId: str | None = None
    """
The Amazon Resource Name (`key ARN <https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#key-id-key-ARN>`_)
of the KMS key whose deletion is scheduled.
    """
    DeletionDate: datetime | None = None
    """
    The date and time after which KMS deletes the KMS key.
    """
    KeyState: Literal["Creating", "Enabled", "Disabled", "PendingDeletion", "PendingImport", "PendingReplicaDeletion", "Unavailable", "Updating"] | None = None
    """
    The current status of the KMS key.
    """
    PendingWindowInDays: int | None = None
    """
    The waiting period before the KMS key is deleted.
    """


class CancelKeyDeletionResponse(Boto3Model):
    KeyId: str | None = None
    """
The Amazon Resource Name (`key ARN <https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#key-id-key-ARN>`_)
of the KMS key whose deletion is canceled.
    """



# This file is automatically generated by botocraft.  Do not edit directly.
# mypy: disable-error-code="index, override, assignment, union-attr, misc"
from botocraft.eventbridge.factory import AbstractEventFactory, EventFactory
from botocraft.mixins.tags import TagsDictMixin
from botocraft.mixins.sqs import queue_recieve_messages_add_queue_url
from collections import OrderedDict
from botocraft.mixins.sqs import queue_recieve_messages_add_event_factory
from typing import ClassVar, Type, Optional, Literal, Dict, List, Union, Any, cast
from datetime import datetime
from pydantic import Field
from .common import Tag
from botocraft.mixins.sqs import QueueManagerMixin
from botocraft.mixins.sqs import MessageModelMixin
from .abstract import PrimaryBoto3ModelQuerySet
from botocraft.mixins.sqs import QueueModelMixin
from .abstract import (
    Boto3Model,
    ReadonlyBoto3Model,
    PrimaryBoto3Model,
    ReadonlyPrimaryBoto3Model,
    Boto3ModelManager,
    ReadonlyBoto3ModelManager,
)
from botocraft.mixins.sqs import queue_list_urls_to_queues

# ===============
# Managers
# ===============


class QueueManager(QueueManagerMixin, Boto3ModelManager):
    service_name: str = "sqs"

    def create(self, model: "Queue") -> str:
        """
        Creates a new standard or FIFO queue. You can pass one or more attributes in the
        request. Keep the following in mind:

        Args:
            model: The :py:class:`Queue` to create.
        """
        data = model.model_dump(exclude_none=True, by_alias=True)
        args = dict(
            QueueName=data.get("QueueName"),
            Attributes=data.get("Attributes"),
            tags=data.get("tags"),
        )
        _response = self.client.create_queue(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = CreateQueueResult(**_response)

        self.sessionize(response.QueueUrl)
        return cast("str", response.QueueUrl)

    def delete(self, QueueUrl: str) -> None:
        """
        Deletes the queue specified by the ``QueueUrl``, regardless of the queue's
        contents.

        Args:
            QueueUrl: The URL of the Amazon SQS queue to delete.
        """
        args: Dict[str, Any] = dict(QueueUrl=self.serialize(QueueUrl))
        self.client.delete_queue(**{k: v for k, v in args.items() if v is not None})

    @queue_list_urls_to_queues
    def list(self, *, QueueNamePrefix: str | None = None) -> List[str]:
        """
        Returns a list of your queues in the current region. The response includes a
        maximum of 1,000 results. If you specify a value for the optional
        ``QueueNamePrefix`` parameter, only queues with a name that begins with the
        specified value are returned.

        Keyword Args:
            QueueNamePrefix: A string to use for filtering the list results. Only those queues whose name begins with the
                specified string are returned.
        """
        paginator = self.client.get_paginator("list_queues")
        args: Dict[str, Any] = dict(QueueNamePrefix=self.serialize(QueueNamePrefix))
        response_iterator = paginator.paginate(
            **{k: v for k, v in args.items() if v is not None}
        )
        results = []
        for _response in response_iterator:
            if list(_response.keys()) == ["ResponseMetadata"]:
                break
            if "ResponseMetadata" in _response:
                del _response["ResponseMetadata"]
            response = ListQueuesResult(**_response)
            if response.QueueUrls:
                results.extend(response.QueueUrls)
            else:
                if getattr(response, "NextToken", None):
                    continue
                break
        self.sessionize(results)
        if results and isinstance(results[0], Boto3Model):
            return PrimaryBoto3ModelQuerySet(results)
        return results

    def purge(self, QueueUrl: str) -> "None":
        """
        Deletes available messages in a queue (including in-flight messages) specified
        by the ``QueueURL`` parameter.

        Args:
            QueueUrl: The URL of the queue from which the ``PurgeQueue`` action deletes messages.
        """
        args: Dict[str, Any] = dict(QueueUrl=self.serialize(QueueUrl))
        self.client.purge_queue(**{k: v for k, v in args.items() if v is not None})

    def add_permission(
        self, QueueUrl: str, Label: str, AWSAccountIds: List[str], Actions: List[str]
    ) -> "None":
        """
        Adds a permission to a queue for a specific `principal <https://docs.aws.amazon.com/general/latest/gr/glos-chap.html#P>`_.
        This allows sharing access to the queue.

        Args:
            QueueUrl: The URL of the Amazon SQS queue to which permissions are added.
            Label: The unique identification of the permission you're setting (for example, ``AliceSendMessage``). Maximum 80
                characters. Allowed characters include alphanumeric characters, hyphens (``-``), and underscores (``_``).
            AWSAccountIds: The Amazon Web Services account numbers of the `principals
                <https://docs.aws.amazon.com/general/latest/gr/glos- chap.html#P>`_ who are to receive permission. For information
                about locating the Amazon Web Services account identification, see `Your Amazon Web Services Identifiers
                <https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-making-api- requests.html#sqs-api-
                request-authentication>`_ in the *Amazon SQS Developer Guide*.
            Actions: The action the client wants to allow for the specified principal. Valid values: the name of any action or
                ``*``.

        """
        args: Dict[str, Any] = dict(
            QueueUrl=self.serialize(QueueUrl),
            Label=self.serialize(Label),
            AWSAccountIds=self.serialize(AWSAccountIds),
            Actions=self.serialize(Actions),
        )
        self.client.add_permission(**{k: v for k, v in args.items() if v is not None})

    def remove_permission(self, QueueUrl: str, Label: str) -> "None":
        """
        Revokes any permissions in the queue policy that matches the specified ``Label``
        parameter.

        Args:
            QueueUrl: The URL of the Amazon SQS queue from which permissions are removed.
            Label: The identification of the permission to remove. This is the label added using the  ``AddPermission``  action.
        """
        args: Dict[str, Any] = dict(
            QueueUrl=self.serialize(QueueUrl), Label=self.serialize(Label)
        )
        self.client.remove_permission(
            **{k: v for k, v in args.items() if v is not None}
        )

    def delete_message(self, QueueUrl: str, ReceiptHandle: str) -> "None":
        """
        Deletes the specified message from the specified queue. To select the message to
        delete, use the ``ReceiptHandle`` of the message (*not* the ``MessageId`` which
        you receive when you send the message). Amazon SQS can delete a message from a
        queue even if a visibility timeout setting causes the message to be locked by
        another consumer. Amazon SQS automatically deletes messages left in a queue
        longer than the retention period configured for the queue.

        Args:
            QueueUrl: The URL of the Amazon SQS queue from which messages are deleted.
            ReceiptHandle: The receipt handle associated with the message to delete.
        """
        args: Dict[str, Any] = dict(
            QueueUrl=self.serialize(QueueUrl),
            ReceiptHandle=self.serialize(ReceiptHandle),
        )
        self.client.delete_message(**{k: v for k, v in args.items() if v is not None})

    @queue_recieve_messages_add_queue_url
    @queue_recieve_messages_add_event_factory
    def receive_messages(
        self,
        QueueUrl: str,
        *,
        AttributeNames: List[Literal["All", "Policy", "VisibilityTimeout", "MaximumMessageSize", "MessageRetentionPeriod", "ApproximateNumberOfMessages", "ApproximateNumberOfMessagesNotVisible", "CreatedTimestamp", "LastModifiedTimestamp", "QueueArn", "ApproximateNumberOfMessagesDelayed", "DelaySeconds", "ReceiveMessageWaitTimeSeconds", "RedrivePolicy", "FifoQueue", "ContentBasedDeduplication", "KmsMasterKeyId", "KmsDataKeyReusePeriodSeconds", "DeduplicationScope", "FifoThroughputLimit", "RedriveAllowPolicy", "SqsManagedSseEnabled"]] | None = None,
        MessageSystemAttributeNames: List[Literal["All", "SenderId", "SentTimestamp", "ApproximateReceiveCount", "ApproximateFirstReceiveTimestamp", "SequenceNumber", "MessageDeduplicationId", "MessageGroupId", "AWSTraceHeader", "DeadLetterQueueSourceArn"]] | None = None,
        MessageAttributeNames: List[str] | None = None,
        MaxNumberOfMessages: int | None = None,
        VisibilityTimeout: int | None = None,
        WaitTimeSeconds: int | None = None,
        ReceiveRequestAttemptId: str | None = None,
        EventFactoryClass: Type[AbstractEventFactory] | None = None,
    ) -> List["Message"]:
        """
        Retrieves one or more messages (up to 10), from the specified queue. Using the ``WaitTimeSeconds`` parameter enables
        long-poll support. For more information, see `Amazon SQS Long
        Polling <https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-long-polling.html>`_ in the
        *Amazon SQS Developer Guide*.

        Args:
            QueueUrl: The URL of the Amazon SQS queue from which messages are received.

        Keyword Args:
            AttributeNames: This parameter has been discontinued but will be supported for backward compatibility. To provide
                attribute names, you are encouraged to use ``MessageSystemAttributeNames``.
            MessageSystemAttributeNames: A list of attributes that need to be returned along with each message. These attributes
                include:
            MessageAttributeNames: The name of the message attribute, where *N* is the index.
            MaxNumberOfMessages: The maximum number of messages to return. Amazon SQS never returns more messages than this
                value (however, fewer messages might be returned). Valid values: 1 to 10. Default: 1.
            VisibilityTimeout: The duration (in seconds) that the received messages are hidden from subsequent retrieve requests
                after being retrieved by a ``ReceiveMessage`` request. If not specified, the default visibility timeout for the
                queue is used, which is 30 seconds.
            WaitTimeSeconds: The duration (in seconds) for which the call waits for a message to arrive in the queue before
                returning. If a message is available, the call returns sooner than ``WaitTimeSeconds``. If no messages are available
                and the wait time expires, the call does not return a message list. If you are using the Java SDK, it returns a
                ``ReceiveMessageResponse`` object, which has a empty list instead of a Null object.
            ReceiveRequestAttemptId: This parameter applies only to FIFO (first-in-first-out) queues.
            EventFactoryClass: the value to set for EventFactoryClass

        """
        args: Dict[str, Any] = dict(
            QueueUrl=self.serialize(QueueUrl),
            AttributeNames=self.serialize(AttributeNames),
            MessageSystemAttributeNames=self.serialize(MessageSystemAttributeNames),
            MessageAttributeNames=self.serialize(MessageAttributeNames),
            MaxNumberOfMessages=self.serialize(MaxNumberOfMessages),
            VisibilityTimeout=self.serialize(VisibilityTimeout),
            WaitTimeSeconds=self.serialize(WaitTimeSeconds),
            ReceiveRequestAttemptId=self.serialize(ReceiveRequestAttemptId),
        )
        _response = self.client.receive_message(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ReceiveMessageResult(**_response)

        results: List["Message"] = None
        if response is not None:
            results = response.Messages

        self.sessionize(results)
        return cast("List[Message]", results)

    def batch_send_messages(
        self, QueueUrl: str, Entries: List["SendMessageBatchRequestEntry"]
    ) -> "SendMessageBatchResult":
        """
        You can use ``SendMessageBatch`` to send up to 10 messages to the specified
        queue by assigning either identical or different values to each message (or by
        not assigning values at all). This is a batch version of  ``SendMessage.`` For a
        FIFO queue, multiple messages within a single batch are enqueued in the order
        they are sent.

        Args:
            QueueUrl: The URL of the Amazon SQS queue to which batched messages are sent.
            Entries: A list of  ``SendMessageBatchRequestEntry``  items.
        """
        args: Dict[str, Any] = dict(
            QueueUrl=self.serialize(QueueUrl), Entries=self.serialize(Entries)
        )
        _response = self.client.send_message_batch(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = SendMessageBatchResult(**_response)

        results: "SendMessageBatchResult" = None
        if response is not None:
            results = response

        self.sessionize(results)
        return cast("SendMessageBatchResult", results)

    def batch_delete_messages(
        self, QueueUrl: str, Entries: List["DeleteMessageBatchRequestEntry"]
    ) -> "DeleteMessageBatchResult":
        """
        Deletes up to ten messages from the specified queue. This is a batch version of
        ``DeleteMessage.`` The result of the action on each message is reported
        individually in the response.

        Args:
            QueueUrl: The URL of the Amazon SQS queue from which messages are deleted.
            Entries: Lists the receipt handles for the messages to be deleted.
        """
        args: Dict[str, Any] = dict(
            QueueUrl=self.serialize(QueueUrl), Entries=self.serialize(Entries)
        )
        _response = self.client.delete_message_batch(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = DeleteMessageBatchResult(**_response)

        results: "DeleteMessageBatchResult" = None
        if response is not None:
            results = response

        self.sessionize(results)
        return cast("DeleteMessageBatchResult", results)

    def change_message_visibility(
        self, QueueUrl: str, ReceiptHandle: str, VisibilityTimeout: int
    ) -> "None":
        """
        Changes the visibility timeout of a specified message in a queue to a new value. The default visibility timeout for a
        message is 30 seconds. The minimum is 0 seconds. The maximum is 12 hours. For more information, see `Visibility
        Timeout <https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-visibility-timeout.html>`_ in the
        *Amazon SQS Developer Guide*.

        Args:
            QueueUrl: The URL of the Amazon SQS queue whose message's visibility is changed.
            ReceiptHandle: The receipt handle associated with the message, whose visibility timeout is changed. This parameter
                is returned by the ``ReceiveMessage``  action.
            VisibilityTimeout: The new value for the message's visibility timeout (in seconds). Values range: ``0`` to
                ``43200``. Maximum: 12 hours.

        """
        args: Dict[str, Any] = dict(
            QueueUrl=self.serialize(QueueUrl),
            ReceiptHandle=self.serialize(ReceiptHandle),
            VisibilityTimeout=self.serialize(VisibilityTimeout),
        )
        self.client.change_message_visibility(
            **{k: v for k, v in args.items() if v is not None}
        )

    def change_message_visibility_batch(
        self, QueueUrl: str, Entries: List["ChangeMessageVisibilityBatchRequestEntry"]
    ) -> "ChangeMessageVisibilityBatchResult":
        """
        Changes the visibility timeout of multiple messages. This is a batch version of
        ``ChangeMessageVisibility.`` The result of the action on each message is
        reported individually in the response. You can send up to 10
        ``ChangeMessageVisibility``  requests with each ``ChangeMessageVisibilityBatch``
        action.

        Args:
            QueueUrl: The URL of the Amazon SQS queue whose messages' visibility is changed.
            Entries: Lists the receipt handles of the messages for which the visibility timeout must be changed.
        """
        args: Dict[str, Any] = dict(
            QueueUrl=self.serialize(QueueUrl), Entries=self.serialize(Entries)
        )
        _response = self.client.change_message_visibility_batch(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = ChangeMessageVisibilityBatchResult(**_response)

        results: "ChangeMessageVisibilityBatchResult" = None
        if response is not None:
            results = response

        self.sessionize(results)
        return cast("ChangeMessageVisibilityBatchResult", results)


class MessageManager(Boto3ModelManager):
    service_name: str = "sqs"

    def send(
        self,
        QueueUrl: str,
        MessageBody: str,
        *,
        DelaySeconds: int | None = None,
        MessageAttributes: Dict[str, "MessageAttributeValue"] | None = None,
        MessageSystemAttributes: Dict[Literal["AWSTraceHeader"], "MessageSystemAttributeValue"] | None = None,
        MessageDeduplicationId: str | None = None,
        MessageGroupId: str | None = None,
    ) -> str:
        """
        Delivers a message to the specified queue.

        Args:
            QueueUrl: The URL of the Amazon SQS queue to which a message is sent.
            MessageBody: The message to send. The minimum size is one character. The maximum size is 1 MiB or 1,048,576 bytes

        Keyword Args:
            DelaySeconds: The length of time, in seconds, for which to delay a specific message. Valid values: 0 to 900.
                Maximum: 15 minutes. Messages with a positive ``DelaySeconds`` value become available for processing after the delay
                period is finished. If you don't specify a value, the default value for the queue applies.
            MessageAttributes: Each message attribute consists of a ``Name``, ``Type``, and ``Value``. For more information, see
                `Amazon SQS message attributes <https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-
                message-metadata.html#sqs- message-attributes>`_ in the *Amazon SQS Developer Guide*.
            MessageSystemAttributes: The message system attribute to send. Each message system attribute consists of a ``Name``,
                ``Type``, and ``Value``.
            MessageDeduplicationId: This parameter applies only to FIFO (first-in-first-out) queues.
            MessageGroupId: ``MessageGroupId`` is an attribute used in Amazon SQS FIFO (First-In-First-Out) and standard queues.
                In FIFO queues, ``MessageGroupId`` organizes messages into distinct groups. Messages within the same message group
                are always processed one at a time, in strict order, ensuring that no two messages from the same group are processed
                simultaneously. In standard queues, using ``MessageGroupId`` enables fair queues. It is used to identify the tenant
                a message belongs to, helping maintain consistent message dwell time across all tenants during noisy neighbor
                events. Unlike FIFO queues, messages with the same ``MessageGroupId`` can be processed in parallel, maintaining the
                high throughput of standard queues.
        """
        args: Dict[str, Any] = dict(
            QueueUrl=self.serialize(QueueUrl),
            MessageBody=self.serialize(MessageBody),
            DelaySeconds=self.serialize(DelaySeconds),
            MessageAttributes=self.serialize(MessageAttributes),
            MessageSystemAttributes=self.serialize(MessageSystemAttributes),
            MessageDeduplicationId=self.serialize(MessageDeduplicationId),
            MessageGroupId=self.serialize(MessageGroupId),
        )
        _response = self.client.send_message(
            **{k: v for k, v in args.items() if v is not None}
        )
        response = SendMessageResult(**_response)

        results: str = None
        if response is not None:
            results = response.MessageId

        self.sessionize(results)
        return cast("str", results)

    def delete(self, QueueUrl: str, ReceiptHandle: str) -> None:
        """
        Deletes the specified message from the specified queue. To select the message to
        delete, use the ``ReceiptHandle`` of the message (*not* the ``MessageId`` which
        you receive when you send the message). Amazon SQS can delete a message from a
        queue even if a visibility timeout setting causes the message to be locked by
        another consumer. Amazon SQS automatically deletes messages left in a queue
        longer than the retention period configured for the queue.

        Args:
            QueueUrl: The URL of the Amazon SQS queue from which messages are deleted.
            ReceiptHandle: The receipt handle associated with the message to delete.
        """
        args: Dict[str, Any] = dict(
            QueueUrl=self.serialize(QueueUrl),
            ReceiptHandle=self.serialize(ReceiptHandle),
        )
        self.client.delete_message(**{k: v for k, v in args.items() if v is not None})


# ==============
# Service Models
# ==============


class Queue(TagsDictMixin, QueueModelMixin, PrimaryBoto3Model):
    tag_class: ClassVar[Type] = Tag
    manager_class: ClassVar[Type[Boto3ModelManager]] = QueueManager

    QueueUrl: str
    """
    The URL of the queue.

    This is a unique identifier for the queue.
    """
    QueueName: str
    """
    The name of the queue.

    This is a unique identifier for the queue.
    """
    Attributes: Dict[str, str] | None = None
    """
    The attributes of the queue.

    This is a dictionary of attribute names and values.
    """
    Tags: List[Tag] = Field(default_factory=list, alias="tags")
    """
    The tags of the queue.

    This is a list of Tag objects.
    """

    @property
    def pk(self) -> str | None:
        """
        Return the primary key of the model.   This is the value of the
        :py:attr:`QueueUrl` attribute.

        Returns:
            The primary key of the model instance.
        """
        return self.QueueUrl

    @property
    def name(self) -> str | None:
        """
        Return the name of the model.   This is the value of the :py:attr:`QueueName`
        attribute.

        Returns:
            The name of the model instance.
        """
        return self.QueueName

    def __hash__(self) -> int:
        """
        Return the hash of the model.

        This is the value of the
        :py:attr:`QueueUrl` attribute.
        """
        return hash(self.QueueUrl)

    def purge(self) -> "None":
        """
        Purge the queue.
        """

        return (
            cast("QueueManager", self.objects)  # type: ignore[attr-defined]
            .using(self.session)
            .purge(
                cast("str", self.QueueUrl),
            )
        )

    def add_permission(
        self, Label: str, AWSAccountIds: List[str], Actions: List[str]
    ) -> "None":
        """
        Add permission to the queue.

        Args:
            Label: The unique identification of the permission you're setting (for example, ``AliceSendMessage``). Maximum 80
                characters. Allowed characters include alphanumeric characters, hyphens (``-``), and underscores (``_``).
            AWSAccountIds: The Amazon Web Services account numbers of the `principals
                <https://docs.aws.amazon.com/general/latest/gr/glos- chap.html#P>`_ who are to receive permission. For information
                about locating the Amazon Web Services account identification, see `Your Amazon Web Services Identifiers
                <https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-making-api- requests.html#sqs-api-
                request-authentication>`_ in the *Amazon SQS Developer Guide*.
            Actions: The action the client wants to allow for the specified principal. Valid values: the name of any action or
                ``*``.
        """

        return (
            cast("QueueManager", self.objects)  # type: ignore[attr-defined]
            .using(self.session)
            .add_permission(
                cast("str", self.QueueUrl),
                Label,
                AWSAccountIds,
                Actions,
            )
        )

    def remove_permission(self, Label: str) -> "None":
        """
        Remove permission from the queue.

        Args:
            Label: The identification of the permission to remove. This is the label added using the  ``AddPermission``  action.
        """

        return (
            cast("QueueManager", self.objects)  # type: ignore[attr-defined]
            .using(self.session)
            .remove_permission(
                cast("str", self.QueueUrl),
                Label,
            )
        )

    def receive(
        self,
        MessageAttributeNames: List[str] | None = None,
        MaxNumberOfMessages: int | None = None,
        VisibilityTimeout: int | None = None,
        WaitTimeSeconds: int | None = None,
        ReceiveRequestAttemptId: str | None = None,
        EventFactoryClass: Type[AbstractEventFactory] | None = None,
    ) -> List["Message"]:
        """
        Receive messages from the queue.

        Keyword Args:
            MessageAttributeNames: The name of the message attribute, where *N* is the index.
            MaxNumberOfMessages: The maximum number of messages to return. Amazon SQS never returns more messages than this
                value (however, fewer messages might be returned). Valid values: 1 to 10. Default: 1.
            VisibilityTimeout: The duration (in seconds) that the received messages are hidden from subsequent retrieve requests
                after being retrieved by a ``ReceiveMessage`` request. If not specified, the default visibility timeout for the
                queue is used, which is 30 seconds.
            WaitTimeSeconds: The duration (in seconds) for which the call waits for a message to arrive in the queue before
                returning. If a message is available, the call returns sooner than ``WaitTimeSeconds``. If no messages are available
                and the wait time expires, the call does not return a message list. If you are using the Java SDK, it returns a
                ``ReceiveMessageResponse`` object, which has a empty list instead of a Null object.
            ReceiveRequestAttemptId: This parameter applies only to FIFO (first-in-first-out) queues.
            EventFactoryClass: the value to set for EventFactoryClass
        """

        return (
            cast("QueueManager", self.objects)  # type: ignore[attr-defined]
            .using(self.session)
            .receive_messages(
                cast("str", self.QueueUrl),
                MessageAttributeNames=MessageAttributeNames,
                MaxNumberOfMessages=MaxNumberOfMessages,
                VisibilityTimeout=VisibilityTimeout,
                WaitTimeSeconds=WaitTimeSeconds,
                ReceiveRequestAttemptId=ReceiveRequestAttemptId,
                EventFactoryClass=EventFactoryClass,
            )
        )


class MessageAttributeValue(Boto3Model):
    """
    The user-specified message attribute value. For string data types, the ``Value``
    attribute has the same restrictions on the content as the message body. For more
    information, see  ``SendMessage.``

    ``Name``, ``type``, ``value`` and the message body must not be empty or null. All parts of the message attribute,
    including ``Name``, ``Type``, and ``Value``, are part of the message size restriction (1 MiB or 1,048,576 bytes).
    """

    StringValue: str | None = None
    """
    Strings are Unicode with UTF-8 binary encoding.

    For a list of code values, see
    `ASCII Printable Characters <http://en.wikipedia.org/wiki/ASCII#ASCII_printable_characters>`_.
    """
    BinaryValue: bytes | None = None
    """
    Binary type attributes can store any binary data, such as compressed data, encrypted
    data, or images.
    """
    StringListValues: List[str] | None = None
    """
    Not implemented.

    Reserved for future use.
    """
    BinaryListValues: List[bytes] | None = None
    """
    Not implemented.

    Reserved for future use.
    """
    DataType: str
    """
Amazon SQS supports the following logical data types: ``String``, ``Number``, and ``Binary``. For the ``Number`` data
type, you must use ``StringValue``.
    """


class MessageSystemAttributeValue(Boto3Model):
    """
    The user-specified message system attribute value. For string data types, the
    ``Value`` attribute has the same restrictions on the content as the message body.
    For more information, see  ``SendMessage.``

    ``Name``, ``type``, ``value`` and the message body must not be empty or null.
    """

    StringValue: str | None = None
    """
    Strings are Unicode with UTF-8 binary encoding.

    For a list of code values, see
    `ASCII Printable Characters <http://en.wikipedia.org/wiki/ASCII#ASCII_printable_characters>`_.
    """
    BinaryValue: bytes | None = None
    """
    Binary type attributes can store any binary data, such as compressed data, encrypted
    data, or images.
    """
    StringListValues: List[str] | None = None
    """
    Not implemented.

    Reserved for future use.
    """
    BinaryListValues: List[bytes] | None = None
    """
    Not implemented.

    Reserved for future use.
    """
    DataType: str
    """
Amazon SQS supports the following logical data types: ``String``, ``Number``, and ``Binary``. For the ``Number`` data
type, you must use ``StringValue``.
    """


class Message(MessageModelMixin, PrimaryBoto3Model):
    """
    An Amazon SQS message.
    """

    manager_class: ClassVar[Type[Boto3ModelManager]] = MessageManager

    MessageId: str | None = None
    """
    A unique identifier for the message.

    A ``MessageId``is considered unique across all Amazon Web Services accounts for an
    extended period of time.
    """
    ReceiptHandle: str | None = None
    """
    An identifier associated with the act of receiving the message.

    A new receipt handle is returned every time you receive a message. When deleting a
    message, you provide the last received receipt handle to delete the message.
    """
    MD5OfBody: str | None = None
    """
    An MD5 digest of the non-URL-encoded message body string.
    """
    Body: str | None = None
    """
    The message's contents (not URL-encoded).
    """
    Attributes: Dict[Literal["All", "SenderId", "SentTimestamp", "ApproximateReceiveCount", "ApproximateFirstReceiveTimestamp", "SequenceNumber", "MessageDeduplicationId", "MessageGroupId", "AWSTraceHeader", "DeadLetterQueueSourceArn"], str] | None = None
    """
    A map of the attributes requested in  ``ReceiveMessage``  to their respective
    values.

    Supported attributes:
    """
    MD5OfMessageAttributes: str | None = None
    """
    An MD5 digest of the non-URL-encoded message attribute string.

    You can use this attribute to verify that Amazon SQS received the message correctly.
    Amazon SQS URL-decodes the message before creating the MD5 digest. For information
    about MD5, see
    `RFC1321 <https://www.ietf.org/rfc/rfc1321.txt>`_.
    """
    MessageAttributes: Dict[str, "MessageAttributeValue"] | None = None
    """
    Each message attribute consists of a ``Name``, ``Type``, and ``Value``.

    For more information, see
    `Amazon SQS message attributes <https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-message-metadata.html#sqs-
    message-attributes>`_ in the *Amazon SQS Developer Guide*.
    """
    QueueUrl: str | None = None
    """
    The URL of the queue.

    This is a unique identifier for the queue.
    """
    EventFactoryClass: Type[AbstractEventFactory] | None = EventFactory
    """
    The factory class for the event.

    This is used to create events from messages received from the queue.
    """

    @property
    def pk(self) -> OrderedDict[str, Any]:
        """
        The primary key of the message.

        This can be used to delete the message from its queue with the :py:meth:`botocraft.services.sqs.MessageManager.delete` method.
        """

        return OrderedDict(
            {
                "QueueUrl": self.QueueUrl,
                "ReceiptHandle": self.ReceiptHandle,
            }
        )

    def send(
        self,
        DelaySeconds: int | None = None,
        MessageDeduplicationId: str | None = None,
        MessageGroupId: str | None = None,
    ) -> "str":
        """
        Send this message to the queue.

        Keyword Args:
            DelaySeconds: The length of time, in seconds, for which to delay a specific message. Valid values: 0 to 900.
                Maximum: 15 minutes. Messages with a positive ``DelaySeconds`` value become available for processing after the delay
                period is finished. If you don't specify a value, the default value for the queue applies.
            MessageDeduplicationId: This parameter applies only to FIFO (first-in-first-out) queues.
            MessageGroupId: ``MessageGroupId`` is an attribute used in Amazon SQS FIFO (First-In-First-Out) and standard queues.
                In FIFO queues, ``MessageGroupId`` organizes messages into distinct groups. Messages within the same message group
                are always processed one at a time, in strict order, ensuring that no two messages from the same group are processed
                simultaneously. In standard queues, using ``MessageGroupId`` enables fair queues. It is used to identify the tenant
                a message belongs to, helping maintain consistent message dwell time across all tenants during noisy neighbor
                events. Unlike FIFO queues, messages with the same ``MessageGroupId`` can be processed in parallel, maintaining the
                high throughput of standard queues.
        """

        return (
            cast("MessageManager", self.objects)  # type: ignore[attr-defined]
            .using(self.session)
            .send(
                cast("str", self.QueueUrl),
                cast("str", self.MessageBody),
                MessageAttributes=cast(
                    "Dict[str, MessageAttributeValue] | None", self.MessageAttributes
                ),
                DelaySeconds=DelaySeconds,
                MessageDeduplicationId=MessageDeduplicationId,
                MessageGroupId=MessageGroupId,
            )
        )


# =======================
# Request/Response Models
# =======================


class CreateQueueResult(Boto3Model):
    """
    Returns the ``QueueUrl`` attribute of the created queue.
    """

    QueueUrl: str | None = None
    """
    The URL of the created Amazon SQS queue.
    """


class ListQueuesResult(Boto3Model):
    """
    A list of your queues.
    """

    QueueUrls: List[str] | None = None
    """
    A list of queue URLs, up to 1,000 entries, or the value of ``MaxResults`` that you
    sent in the request.
    """
    NextToken: str | None = None
    """
    Pagination token to include in the next request.

    Token value is ``null`` if there are no additional results to request,
    or if you did not set ``MaxResults`` in the request.
    """


class ReceiveMessageResult(Boto3Model):
    """
    A list of received messages.
    """

    Messages: List["Message"] | None = None
    """
    A list of messages.
    """


class SendMessageBatchRequestEntry(Boto3Model):
    """
    Contains the details of a single Amazon SQS message along with an ``Id``.
    """

    Id: str
    """
    An identifier for a message in this batch used to communicate the result.
    """
    MessageBody: str
    """
    The body of the message.
    """
    DelaySeconds: int | None = None
    """
    The length of time, in seconds, for which a specific message is delayed.

    Valid values: 0 to 900. Maximum: 15 minutes.
    Messages with a positive ``DelaySeconds`` value become available for processing after the delay period is finished. If
    you don't specify a value, the default value for the queue is applied.
    """
    MessageAttributes: Dict[str, "MessageAttributeValue"] | None = None
    """
    Each message attribute consists of a ``Name``, ``Type``, and ``Value``.

    For more information, see
    `Amazon SQS message attributes <https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-message-metadata.html#sqs-
    message-attributes>`_ in the *Amazon SQS Developer Guide*.
    """
    MessageSystemAttributes: Dict[Literal["AWSTraceHeader"], "MessageSystemAttributeValue"] | None = None
    """
    The message system attribute to send Each message system attribute consists of a
    ``Name``, ``Type``, and ``Value``.
    """
    MessageDeduplicationId: str | None = None
    """
    This parameter applies only to FIFO (first-in-first-out) queues.
    """
    MessageGroupId: str | None = None
    """
    ``MessageGroupId`` is an attribute used in Amazon SQS FIFO (First-In-First-Out) and
    standard queues.

    In FIFO queues,
    ``MessageGroupId`` organizes messages into distinct groups. Messages within the same message group are always processed
    one at a time, in strict order, ensuring that no two messages from the same group are processed simultaneously. In
    standard queues, using ``MessageGroupId`` enables fair queues. It is used to identify the tenant a message belongs to,
    helping maintain consistent message dwell time across all tenants during noisy neighbor events. Unlike FIFO queues,
    messages with the same ``MessageGroupId`` can be processed in parallel, maintaining the high throughput of standard
    queues.
    """


class SendMessageBatchResultEntry(Boto3Model):
    """
    Encloses a ``MessageId`` for a successfully-enqueued message in a
    ``SendMessageBatch.``
    """

    Id: str
    """
    An identifier for the message in this batch.
    """
    MessageId: str
    """
    An identifier for the message.
    """
    MD5OfMessageBody: str
    """
    An MD5 digest of the non-URL-encoded message body string.

    You can use this attribute to verify that Amazon SQS received the message correctly.
    Amazon SQS URL-decodes the message before creating the MD5 digest. For information
    about MD5, see
    `RFC1321 <https://www.ietf.org/rfc/rfc1321.txt>`_.
    """
    MD5OfMessageAttributes: str | None = None
    """
    An MD5 digest of the non-URL-encoded message attribute string.

    You can use this attribute to verify that Amazon SQS received the message correctly.
    Amazon SQS URL-decodes the message before creating the MD5 digest. For information
    about MD5, see
    `RFC1321 <https://www.ietf.org/rfc/rfc1321.txt>`_.
    """
    MD5OfMessageSystemAttributes: str | None = None
    """
    An MD5 digest of the non-URL-encoded message system attribute string.

    You can use this attribute to verify that Amazon SQS received the message correctly.
    Amazon SQS URL-decodes the message before creating the MD5 digest. For information
    about MD5, see
    `RFC1321 <https://www.ietf.org/rfc/rfc1321.txt>`_.
    """
    SequenceNumber: str | None = None
    """
    This parameter applies only to FIFO (first-in-first-out) queues.
    """


class BatchResultErrorEntry(Boto3Model):
    """
    Gives a detailed description of the result of an action on each entry in the
    request.
    """

    Id: str
    """
    The ``Id`` of an entry in a batch request.
    """
    SenderFault: bool
    """
    Specifies whether the error happened due to the caller of the batch API action.
    """
    Code: str
    """
    An error code representing why the action failed on this entry.
    """
    Message: str | None = None
    """
    A message explaining why the action failed on this entry.
    """


class SendMessageBatchResult(Boto3Model):
    """
    For each message in the batch, the response contains a
    ``SendMessageBatchResultEntry``  tag if the message succeeds or a
    ``BatchResultErrorEntry``  tag if the message fails.
    """

    Successful: List["SendMessageBatchResultEntry"]
    """
    A list of  ``SendMessageBatchResultEntry``  items.
    """
    Failed: List["BatchResultErrorEntry"]
    """
    A list of  ``BatchResultErrorEntry``  items with error details about each message
    that can't be enqueued.
    """


class DeleteMessageBatchRequestEntry(Boto3Model):
    """
    Encloses a receipt handle and an identifier for it.
    """

    Id: str
    """
    The identifier for this particular receipt handle.

    This is used to communicate the result.
    """
    ReceiptHandle: str
    """
    A receipt handle.
    """


class DeleteMessageBatchResultEntry(Boto3Model):
    """
    Encloses the ``Id`` of an entry in  ``DeleteMessageBatch.``
    """

    Id: str
    """
    Represents a successfully deleted message.
    """


class DeleteMessageBatchResult(Boto3Model):
    """
    For each message in the batch, the response contains a
    ``DeleteMessageBatchResultEntry``  tag if the message is deleted or a
    ``BatchResultErrorEntry``  tag if the message can't be deleted.
    """

    Successful: List["DeleteMessageBatchResultEntry"]
    """
    A list of  ``DeleteMessageBatchResultEntry``  items.
    """
    Failed: List["BatchResultErrorEntry"]
    """
    A list of  ``BatchResultErrorEntry``  items.
    """


class ChangeMessageVisibilityBatchRequestEntry(Boto3Model):
    """
    Encloses a receipt handle and an entry ID for each message in
    ``ChangeMessageVisibilityBatch.``
    """

    Id: str
    """
    An identifier for this particular receipt handle used to communicate the result.
    """
    ReceiptHandle: str
    """
    A receipt handle.
    """
    VisibilityTimeout: int | None = None
    """
    The new value (in seconds) for the message's visibility timeout.
    """


class ChangeMessageVisibilityBatchResultEntry(Boto3Model):
    """
    Encloses the ``Id`` of an entry in  ``ChangeMessageVisibilityBatch.``
    """

    Id: str
    """
    Represents a message whose visibility timeout has been changed successfully.
    """


class ChangeMessageVisibilityBatchResult(Boto3Model):
    """
    For each message in the batch, the response contains a
    ``ChangeMessageVisibilityBatchResultEntry``  tag if the message succeeds or a
    ``BatchResultErrorEntry``  tag if the message fails.
    """

    Successful: List["ChangeMessageVisibilityBatchResultEntry"]
    """
    A list of  ``ChangeMessageVisibilityBatchResultEntry``  items.
    """
    Failed: List["BatchResultErrorEntry"]
    """
    A list of  ``BatchResultErrorEntry``  items.
    """


class SendMessageResult(Boto3Model):
    """
    The ``MD5OfMessageBody`` and ``MessageId`` elements.
    """

    MD5OfMessageBody: str | None = None
    """
    An MD5 digest of the non-URL-encoded message body string.

    You can use this attribute to verify that Amazon SQS received the message correctly.
    Amazon SQS URL-decodes the message before creating the MD5 digest. For information
    about MD5, see
    `RFC1321 <https://www.ietf.org/rfc/rfc1321.txt>`_.
    """
    MD5OfMessageAttributes: str | None = None
    """
    An MD5 digest of the non-URL-encoded message attribute string.

    You can use this attribute to verify that Amazon SQS received the message correctly.
    Amazon SQS URL-decodes the message before creating the MD5 digest. For information
    about MD5, see
    `RFC1321 <https://www.ietf.org/rfc/rfc1321.txt>`_.
    """
    MD5OfMessageSystemAttributes: str | None = None
    """
    An MD5 digest of the non-URL-encoded message system attribute string.

    You can use this attribute to verify that Amazon SQS received the message correctly.
    Amazon SQS URL-decodes the message before creating the MD5 digest.
    """
    MessageId: str | None = None
    """
    An attribute containing the ``MessageId`` of the message sent to the queue.

    For more information, see
    `Queue and Message Identifiers <https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-queue-message-
    identifiers.html>`_ in the *Amazon SQS Developer Guide*.
    """
    SequenceNumber: str | None = None
    """
    This parameter applies only to FIFO (first-in-first-out) queues.
    """



from __future__ import annotations

from datetime import datetime  # noqa: TC003
from typing import List

from pydantic import BaseModel, Field


class ECSServiceAction(BaseModel):
    """
    The main body of the ECS Service Action event.
    """

    #: A list of ARNs of the capacity providers associated with the service.
    capacityProviderArns: List[str] | None = None
    #: The ARN of the cluster associated with the service.
    clusterArn: str
    #: The time the event was created
    createdAt: datetime | None = None
    #: The name of the event
    eventName: str
    #: The type of the event
    eventType: str
    #: A reason for the event
    reason: str | None = None
    #: The desired count of tasks in the service
    desiredCount: float | None = None
    #: The container port of the service  (What if I have multiple ports?)
    containerPort: float | None = None
    #: The list of task ARNs associated with the service
    taskArns: List[str] | None = None
    #: The list of task set ARNs associated with the service
    taskSetArns: List[str] | None = None
    #: The list of container instance ARNs associated with the service
    containerInstanceArns: List[str] | None = None
    #: The list of EC2 instance IDs associated with the service
    ec2InstanceIds: List[str] | None = None
    #: The list of target group ARNs associated with the service
    targetGroupArns: List[str] | None = None
    #: The list of service registry ARNs associated with the service
    serviceRegistryArns: List[str] | None = None
    #: Not sure what this is
    targets: List[str] | None = None


class ECSServiceActionEvent(BaseModel):
    #: Where the meat of the event is
    detail: ECSServiceAction
    #: The account the event was generated in
    account: str
    #: The human readable name of the event
    detail_type: str = Field(..., alias="detail-type")
    #: The ID of the event
    id: str
    #: The region the event was generated in
    region: str
    #: The resources the event applies to.  This will be a list of ARNs
    resources: List[str]
    #: The source of the event, e.g. "aws.ecs"
    source: str
    #: The time the event was generated, in UTC
    time: datetime
    #: The version of the schema for the event
    version: str




import click

from botocraft.sync.models import BotocraftInterface

from .cli import cli


@cli.command("sync", short_help="Sync an aws service to botocraft")
@click.option("--service", default=None, help="The name of the AWS service to sync")
def models_sync(service: str | None) -> None:
    interface = BotocraftInterface()
    interface.load()
    if service and service not in interface.services:
        msg = f'No service definition for AWS Service "{service}"'
        raise click.ClickException(msg)
    interface.generate(service=service)



from __future__ import annotations

from datetime import datetime  # noqa: TC003
from typing import List

from pydantic import BaseModel, Field


class FindingSeverityCounts(BaseModel):
    """
    Represents the counts of security findings by severity level from an ECR image scan.
    Each attribute indicates the number of findings at that severity level.
    """

    #: Number of critical severity findings detected in the scan
    CRITICAL: float | None = None
    #: Number of high severity findings detected in the scan
    HIGH: float | None = None
    #: Number of informational severity findings detected in the scan
    INFORMATIONAL: float | None = None
    #: Number of low severity findings detected in the scan
    LOW: float | None = None
    #: Number of medium severity findings detected in the scan
    MEDIUM: float | None = None
    #: Number of findings with undefined severity level
    UNDEFINED: float | None = None


class ECRImageScan(BaseModel):
    """
    Represents the details of an ECR image scan operation.
    Contains information about the scanned image and the scan results.
    """

    #: The SHA256 digest of the image that was scanned
    image_digest: str | None = Field(None, alias="image-digest")
    #: The list of tags associated with the scanned image
    image_tags: List[str] | None = Field(None, alias="image-tags")
    #: The name of the ECR repository containing the scanned image
    repository_name: str = Field(..., alias="repository-name")
    #: The status of the scan operation (e.g., "COMPLETE", "FAILED")
    scan_status: str = Field(..., alias="scan-status")
    #: The counts of findings grouped by severity level
    finding_severity_counts: FindingSeverityCounts | None = Field(
        None, alias="finding-severity-counts"
    )


class ECRImageScanEvent(BaseModel):
    """
    Represents a complete ECR Image Scan event from EventBridge.
    Contains metadata about the event and the actual scan details.
    """

    #: The detailed information about the image scan operation
    detail: ECRImageScan
    #: The AWS account ID where the scan was performed
    account: str
    #: The human-readable type of the event (e.g., "ECR Image Scan")
    detail_type: str = Field(..., alias="detail-type")
    #: The unique identifier for this event instance
    id: str
    #: The AWS region where the scan was performed
    region: str
    #: The AWS resources involved in this event, typically the repository ARN
    resources: List[str]
    #: The source of the event (typically "aws.ecr")
    source: str
    #: The timestamp when the scan was completed and event generated, in UTC
    time: datetime
    #: The version of the event schema format
    version: str
